# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""DIS Data v2 action implementations"""
import logging
from pathlib import Path

from osc_lib import utils
# from osc_lib import exceptions
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils
from otcextensions.common import cli_utils


LOG = logging.getLogger(__name__)


_formatters = {
    'timestamp': cli_utils.UnixTimestampFormatter,
    'records': cli_utils.YamlFormat,
}


def _get_columns(item):
    column_map = {}
    hidden = [
        'location',
        'app_id',
    ]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map,
                                                           hidden)


DATA_FILE_TEMPATE = """data,partition_id,partitition_key,explicit_hash_key
TXkgRGF0YQo=,1,2,12345678
My string data,2
MyData"""


CURSOR_TYPE_CHOICES = (
    'AT_SEQUENCE_NUMBER',
    'AFTER_SEQUENCE_NUMBER',
    'TRIM_HORIZON',
    'LATEST',
    'AT_TIMESTAMP',
)


class DownloadData(command.Lister):

    _description = _("Download Data.")

    columns = (
        'sequence_number',
        'data',
        'timestamp',
        'timestamp_type',
    )

    display_columns = (
        'Sequence Number',
        'Data',
        'Timestamp',
        'Timestamp Type',
    )

    def get_parser(self, prog_name):
        parser = super(DownloadData, self).get_parser(prog_name)

        parser.add_argument(
            '--partition-cursor',
            metavar='<partition_cursor>',
            required=True,
            help=_("Data cursor, which needs to be obtained through the API "
                   "for obtaining data cursors. Value: 1 to 512 characters."
                   "\nNote: The validity period of a data cursor is 5 minutes."
                   " Maximum number of apps to list in a single API call."),
        )
        parser.add_argument(
            '--max-fetch-bytes',
            metavar='<max_fetch_bytes>',
            help=_("Maximum number of bytes that can be obtained for each "
                   "request."
                   "\nNote: If the value is less than the size of a single "
                   "record in the partition, the record cannot be obtained."),
        )
        parser.add_argument(
            '--filename',
            metavar='<filename>',
            help=_('To save data to a file.\n'
                   'Note: File exentension should be `csv`'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis

        data = client.download_data(parsed_args.partition_cursor,
                                    parsed_args.max_fetch_bytes,
                                    parsed_args.filename)

        if parsed_args.filename:
            return (self.display_columns, ())
        return (
            self.display_columns,
            (
                utils.get_item_properties(
                    s, self.columns, formatters=_formatters
                ) for s in data
            )
        )


class ShowDataCursor(command.ShowOne):
    _description = _("Query Data Cursor.")

    def get_parser(self, prog_name):
        parser = super(ShowDataCursor, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            help=_("Name of the stream."),
        )
        parser.add_argument(
            '--partition-id',
            metavar='<partition_id>',
            required=True,
            help=_("Partition ID of the stream."),
        )
        parser.add_argument(
            '--cursor-type',
            dest='cursor-type',
            metavar='{' + ','.join(CURSOR_TYPE_CHOICES) + '}',
            type=lambda s: s.upper(),
            choices=CURSOR_TYPE_CHOICES,
            help=_("Cursor Type."),
        )
        parser.add_argument(
            '--starting-seq-num',
            metavar='<starting_seq_num>',
            dest='starting-sequence-number',
            help=_("Sequence number. A sequence number is the unique "
                   "identifier of each record."),
        )
        parser.add_argument(
            '--timestamp',
            metavar='<timestamp>',
            type=int,
            help=_("Timestamp when the data record starts to be read, which "
                   "is closely related to cursor type `AT_TIMESTAMP`."),
        )
        parser.add_argument(
            '--stream-id',
            metavar='<stream_id>',
            dest='stream-id',
            help=_("Unique ID of the stream."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis
        params = {}
        for arg in ('cursor-type',
                    'starting-sequence-number',
                    'timestamp', 'stream-id',):
            val = getattr(parsed_args, arg)
            if val:
                params[arg] = val

        obj = client.get_data_cursor(parsed_args.streamName,
                                     parsed_args.partition_id,
                                     **params)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class UploadData(command.ShowOne):
    _description = _("upload data to DIS streams.")

    def get_parser(self, prog_name):
        parser = super(UploadData, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            help=_("Name of the Stream."),
        )
        parser.add_argument(
            '--stream-id',
            metavar='<stream_id>',
            help=_("Unique ID of the stream. If no stream is found based on "
                   "stream_name and stream_id is not empty, stream_id is "
                   "used to search for the stream."),
        )
        parser.add_argument(
            '--data',
            metavar='<data>',
            help=_("Data to be uploaded. The uploaded data is the serialized "
                   "binary data (character string encoded using Base64)."),
        )
        parser.add_argument(
            '--explicit-hash-key',
            metavar='<explicit_hash_key>',
            help=_("Hash value of the data to be written to the partition. "
                   "The hash value overwrites the hash value of "
                   "partition_key."),
        )
        parser.add_argument(
            '--partition-id',
            metavar='<partition_id>',
            help=_("Partition ID of the stream."),
        )
        parser.add_argument(
            '--partition-key',
            metavar='<partition_key>',
            help=_("Partition to which data is written to."
                   "\nNote:If the partition_id parameter is transferred, "
                   "it will be preferentially used. If partition_id is "
                   "not passed, partition_key will be used."),
        )
        parser.add_argument(
            '--filename',
            metavar='<filename>',
            help=_('Data file path in CSV format.\n'
                   'To get template of a data file run this command:\n'
                   'openstack dis data file template'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis

        attrs = {
            'stream_name': parsed_args.streamName,
            'stream_id': parsed_args.stream_id
        }
        if parsed_args.filename:
            attrs['filename'] = parsed_args.filename

        else:
            records = {}
            for arg in ('data', 'explicit_hash_key',
                        'partition_id', 'partition_key',):
                val = getattr(parsed_args, arg)
                if val:
                    records[arg] = val
            attrs['records'] = [records]

        obj = client.upload_data(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class DataFileTemplate(command.Command):
    _description = _("Print Data File Template.")

    def get_parser(self, prog_name):
        parser = super(DataFileTemplate, self).get_parser(prog_name)
        parser.add_argument(
            '--filename',
            metavar='<filename>',
            required=True,
            help=_('FileName to generate the template.'),
        )
        return parser

    def take_action(self, parsed_args):
        with Path(parsed_args.output_file).open('w') as out_file:
            out_file.write(DATA_FILE_TEMPATE)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.dms.v1 import quota

EXAMPLE_LIST = {
    'quotas': {
        'resources': [
            {
                'type': 'queue',
                'used': 2,
                'quota': 25,
                'min': 1,
                'max': 50
            }
        ]
    }
}


class TestQuota(base.TestCase):

    def setUp(self):
        super(TestQuota, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()

        self.sot = quota.Quota()

    def test_basic(self):
        pass
        sot = quota.Quota()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual('quotas.resources', sot.resources_key)
        self.assertEqual('/quotas/dms', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        obj = EXAMPLE_LIST['quotas']['resources'][0]
        sot = quota.Quota.existing(**obj)
        self.assertEqual(obj['type'], sot.type)
        self.assertEqual(obj['used'], sot.used)
        self.assertEqual(obj['quota'], sot.quota)
        self.assertEqual(obj['min'], sot.min)
        self.assertEqual(obj['max'], sot.max)

    def test_list(self):
        pass
#        sot = quota.Quota()
#        mock_response = mock.Mock()
#        mock_response.status_code = 200
#        mock_response.json.return_value = copy.deepcopy(EXAMPLE_LIST)
#
#        self.sess.get.return_value = mock_response
#
#        result = list(sot.list(self.sess))
#
#        self.sess.get.assert_called_once_with(
#            '/quotas/dms',
#            headers={'Content-Type': 'application/json'},
#            params={}
#        )
#
#        expected_list = [
#            quota.Quota.existing(
#                **EXAMPLE_LIST['quotas']['resources'][0])]
#
#        self.assertEqual(expected_list, result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from urllib.parse import urlparse

from openstack import resource
from openstack.utils import urljoin


class BadBaseResource(resource.Resource):
    """A base class for all terribly exposed non Keystone native extensions
    """

    def __init__(self, _synchronized=False, connection=None, **attrs):
        super(BadBaseResource, self).__init__(
            _synchronized=_synchronized,
            connection=connection, **attrs)
        if connection:
            # Hopefully we land here when creating/fetching resource. If this
            # is the case - try to modify base_url of the resource to be
            # pointing to the FQDN instead of relative URL due to a completely
            # insane service publishing.
            # We need this for the operations, which doesn't support overriding
            # base_path from proxy layer
            identity_url = connection.identity.get_endpoint_data().url
            parsed_domain = urlparse(identity_url)
            self.__base = '%s://%s' % (
                parsed_domain.scheme, parsed_domain.netloc)
            self.base_path = urljoin(
                self.__base,
                self.base_path)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts service v1 action implementations"""
import logging

from cliff import columns as cliff_columns
from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


class ProjectType(cliff_columns.FormattableColumn):
    CHOICES_MAP = {
        0: "non-ExeML project",
        1: "ExeML job for image classification",
        2: "ExeML job for object detection",
        3: "ExeML job for predictive analytics",
    }
    STR = "\n".join(f"{key}: {value}" for key, value in CHOICES_MAP.items())

    def human_readable(self):
        return self.CHOICES_MAP.get(self._value, str(self._value))


class ListJobFlavors(command.Lister):
    _description = _("List Training job resource Specifications.")
    columns = (
        "Spec Id",
        "Spec Code",
        "Core",
        "CPU",
        "GPU Num",
        "GPU Type",
    )

    def get_parser(self, prog_name):
        parser = super(ListJobFlavors, self).get_parser(prog_name)
        parser.add_argument(
            "--job-type",
            metavar="{train, inference}",
            type=lambda s: s.lower(),
            choices=["train", "inference"],
            help=_("Job Type."),
        )
        parser.add_argument(
            "--engine-id",
            metavar="<engine_id>",
            type=int,
            help=_("Engine ID of a job. Default value: 0"),
        )
        parser.add_argument(
            "--project-type",
            metavar="<project_type>",
            choices=list(ProjectType.CHOICES_MAP.keys()),
            type=int,
            help=_("Project type. Default value: 0\n" + ProjectType.STR),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        query_params = {}
        for arg in ("job_type", "engine_id", "project_type"):
            val = getattr(parsed_args, arg)
            if val or str(val) == "0":
                query_params[arg] = val

        data = client.job_flavors(**query_params)

        return (
            self.columns,
            (utils.get_item_properties(s, self.columns) for s in data),
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CTS Trace v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.common import format

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)

TRACE_STATUSES = ['NORMAL', 'WARNING', 'INCIDENT']


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListTrace(command.Lister):
    _description = _('List CTS traces')
    columns = (
        'id', 'name', 'user', 'service_type', 'type',
        'resource_type', 'resource_name', 'resource_id',
        'source_ip', 'level', 'time')

    def get_parser(self, prog_name):
        parser = super(ListTrace, self).get_parser(prog_name)
        parser.add_argument(
            '--tracker',
            metavar='<tracker>',
            default='system',
            help=_('Tracker name (currently only `system`)')
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Limit number of records to return.')
        )
        parser.add_argument(
            '--next',
            metavar='<next>',
            type=int,
            help=_('ID of the previous trace to paginate output.')
        )
        parser.add_argument(
            '--service_type',
            metavar='<type>',
            help=_('Service type to query traces for.')
        )
        parser.add_argument(
            '--resource_type',
            metavar='<resource_type>',
            help=_('Resource type to query traces for.')
        )
        parser.add_argument(
            '--resource_id',
            metavar='<resource_id>',
            help=_('Resource ID to query traces for.')
        )
        parser.add_argument(
            '--resource_name',
            metavar='<resource_name>',
            help=_('Resource name to query traces for.')
        )
        parser.add_argument(
            '--trace_name',
            metavar='<name>',
            help=_('Trace name (operation) to query traces for.')
        )
        parser.add_argument(
            '--trace_id',
            metavar='<id>',
            help=_('Trace id to query traces for.')
        )
        parser.add_argument(
            '--level',
            metavar='{' + ','.join(TRACE_STATUSES) + '}',
            type=lambda s: s.upper(),
            choices=TRACE_STATUSES,
            help=_('Trace level to query traces for.')
        )
        parser.add_argument(
            '--user',
            metavar='<user>',
            help=_('User name to query traces for.')
        )
        parser.add_argument(
            '--start_time',
            metavar='<yyyy-MM-ddTHH:mm:ss>',
            help=_('Start time of the period to be queried.')
        )
        parser.add_argument(
            '--end_time',
            metavar='<yyyy-MM-ddTHH:mm:ss>',
            help=_('End time of the period to be queried.')
        )
        parser.add_argument(
            '--long',
            action='store_true',
            help=_('Return long format.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cts

        query = {}
        if parsed_args.service_type:
            query['service_type'] = parsed_args.service_type
        if parsed_args.resource_type:
            query['res_type'] = parsed_args.resource_type
        if parsed_args.resource_id:
            query['res_id'] = parsed_args.resource_id
        if parsed_args.resource_name:
            query['res_name'] = parsed_args.resource_name
        if parsed_args.trace_name:
            query['trace_name'] = parsed_args.trace_name
        if parsed_args.trace_id:
            query['trace_id'] = parsed_args.trace_id
        if parsed_args.level:
            query['level'] = parsed_args.level
        if parsed_args.user:
            query['user'] = parsed_args.user

        if parsed_args.limit:
            query['limit'] = parsed_args.limit
        if parsed_args.next:
            query['next'] = parsed_args.next
        if parsed_args.start_time:
            query['from'] = format.TimeTMsStr().serialize(
                parsed_args.start_time)
        if parsed_args.end_time:
            query['to'] = format.TimeTMsStr().serialize(
                parsed_args.end_time)

        data = client.traces(
            tracker=parsed_args.tracker,
            **query
        )

        columns = ()
        if not parsed_args.long:
            columns = self.columns
        else:
            columns = (
                'id', 'name', 'type', 'user', 'service_type',
                'resource_type', 'resource_name', 'resource_id',
                'source_ip', 'level', 'time', 'request', 'response'
            )

        table = (columns,
                 (utils.get_item_properties(
                     s, columns,
                 ) for s in data))
        return table



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy
from openstack import resource

from otcextensions.common.utils import extract_url_parts
from otcextensions.sdk.sfsturbo.v1 import share as _sfs
from otcextensions.common.exc import HTTPMethodNotAllowed


class Proxy(proxy.Proxy):

    skip_discovery = True

    def _extract_name(self, url, service_type=None, project_id=None):
        return extract_url_parts(url, project_id)

    # ========== Sfs Turbo ==========
    def shares(self, **query):
        """Return a generator of sfs turbo file systems

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of share objects

        :rtype: :class:`~otcextensions.sdk.sfsturbo.v1.share.Share`
        """
        base_path = _sfs.Share.base_path + '/detail'
        return self._list(_sfs.Share, base_path=base_path, **query)

    def create_share(self, **attrs):
        """Create a new sfs turbo file system

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.sfsturbo.v1.share.Share`
        """
        return self._create(_sfs.Share, **attrs)

    def update_share(self, share, **attrs):
        """Update a new sfs turbo file system
        """
        raise HTTPMethodNotAllowed

    def delete_share(self, share, ignore_missing=True):
        """Delete a sfs turbo file system

        :param share: share id or an instance of
            :class:`~otcextensions.sdk.sfsturbo.v1.share.Share`

        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the vpc route does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent route.

        :returns: none
        """
        return self._delete(_sfs.Share, share,
                            ignore_missing=ignore_missing)

    def get_share(self, share):
        """Get a sfs turbo file system by id

        :param share: share id or an instance of
           :class:`~otcextensions.sdk.sfsturbo.v1.share.Share`

        :returns: One :class:`~otcextensions.sdk.sfsturbo.v1.share.Share`
        """
        return self._get(_sfs.Share, share)

    def find_share(self, name_or_id, ignore_missing=True):
        """Find a single sfs turbo file system by id

        :param name_or_id: The name or ID of a share

        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the vpc does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent peering.

        :returns: One :class:`~otcextensions.sdk.sfsturbo.v1.share.Share`
        """
        return self._find(
            _sfs.Share, name_or_id,
            ignore_missing=ignore_missing,
            list_base_path='/sfs-turbo/shares/detail',
        )

    def wait_for_share(self, share, status='200', failures=None,
                       interval=5, wait=350):
        """Wait for a share to be in a particular status.

        :param share:
            The :class:`~otcextensions.sdk.share.v1.share.Share`
            or share ID to wait on to reach the specified status.
        :param str status: Desired status.
        :param list failures:
            Statuses that would be interpreted as failures.
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait before the change.
            Default to 180
        :return: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
                 to the desired status failed to occur in specified seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
                 has transited to one of the failure statuses.
        """
        failures = failures or ['300', '303']
        return resource.wait_for_status(
            self, share, status, failures, interval, wait)

    def wait_for_extend_capacity(self, share, interval=5, wait=350):
        """Wait for extending capacity.

        :param share: The value can be the
            a :class:`~otcextensions.sdk.sfsturbo.v1.share.Share` instance.
        :param int nterval: Number of seconds to wait between checks.
            Set to ``None`` to use the default interval.
        :param int wait: Maximum number of seconds to wait for transition.
            Set to ``None`` to wait forever.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
                 to the desired status failed to occur in specified seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
                 has transited to one of the failure statuses.
        """
        return share.wait_for_substatus(
            self, desired_substatus='221', failure='321',
            interval=interval, wait=wait)

    def wait_for_change_security_group(self, share, interval=5, wait=300):
        """Wait for changing security group.

        :param share: The value can be the
            a :class:`~otcextensions.sdk.sfsturbo.v1.share.Share` instance.
        :param int interval: Number of seconds to wait between checks.
            Set to ``None`` to use the default interval.
        :param int wait: Maximum number of seconds to wait for transition.
            Set to ``None`` to wait forever.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
                 to the desired status failed to occur in specified seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
                 has transited to one of the failure statuses.
        """
        return share.wait_for_substatus(
            self, desired_substatus='232',
            interval=interval, wait=wait)

    def extend_capacity(self, share, new_size):
        """Extend the capacity of the file system

        :param share: The value can be the ID of a share
             or a :class:`~otcextensions.sdk.sfsturbo.v1.share.Share`
             instance.
        :param new_size: Specifies the new capacity (GB)
            of the shared file system.
        """
        share = self._get_resource(_sfs.Share, share)
        extend = {'new_size': new_size}
        return share.extend_capacity(
            self,
            extend=extend)

    def change_security_group(self, share, security_group_id):
        """Change the security group bound to an SFS Turbo file system.

        :param share: The value can be the ID of a share
             or a :class:`~otcextensions.sdk.sfsturbo.v1.share.Share`
             instance.
        :param security_group_id: Specifies the ID of the security group to be
             modified.
        """
        share = self._get_resource(_sfs.Share, share)
        change_security_group = {'security_group_id': security_group_id}
        return share.change_security_group(
            self,
            change_security_group=change_security_group)

    def wait_for_delete_share(self, share, interval=2, wait=60):
        """Wait for the share to be deleted.

        :param share:
            The :class:`~otcextensions.sdk.sfsturbo.v1.share.Share`
            or share ID to wait on to be deleted.
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait for the delete.
            Default to 60.
        :return: Method returns self on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` transition
                 to status failed to occur in wait seconds.
        """
        share = self._get_resource(_sfs.Share, share)
        return resource.wait_for_delete(self, share, interval, wait)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""DIS Stream v2 action implementations"""
import logging

from osc_lib import utils
from osc_lib import exceptions
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils
from otcextensions.common import cli_utils


LOG = logging.getLogger(__name__)


_formatters = {
    'created_at': cli_utils.UnixTimestampFormatter,
}


def _get_columns(item):
    column_map = {}
    hidden = [
        'location',
        'app_id',
    ]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map,
                                                           hidden)


class ListApps(command.Lister):
    _description = _("Query List of Apps.")

    columns = (
        'name',
        'id',
        'created_at',
    )

    display_columns = (
        'App Name',
        'Id',
        'Created At',
    )

    def get_parser(self, prog_name):
        parser = super(ListApps, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_("Maximum number of apps to list in a single API call. "
                   "\nDefault: 10"),
        )
        parser.add_argument(
            '--start-app-name',
            metavar='<start_app_name>',
            help=_("Name of the app to start the list with. The returned "
                   "app list does not contain this app name."),
        )
        parser.add_argument(
            '--stream-name',
            metavar='<stream_name>',
            help=_("Name of the stream whose apps will be returned."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis
        args_list = (
            'limit',
            'start_app_name',
            'stream_name',
        )
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        data = client.apps(**attrs)

        return (
            self.display_columns,
            (
                utils.get_item_properties(
                    s, self.columns, formatters=_formatters
                ) for s in data
            )
        )


class ListAppConsumptions(command.Lister):
    _description = _("List App Consumptions.")

    columns = (
        'partition_id',
        'sequence_number',
        'checkpoint_type',
    )

    display_columns = (
        'Partition Id',
        'Sequence Number',
        'Checkpoint Type',
    )

    def get_parser(self, prog_name):
        parser = super(ListAppConsumptions, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streaName>',
            help=_("Name of the stream to be queried."),
        )
        parser.add_argument(
            'appName',
            metavar='<appName>',
            help=_("Name of the app to be queried."),
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_("Maximum number of apps to list in a single API call. "
                   "\nDefault: 10"),
        )
        parser.add_argument(
            '--start-partition-id',
            metavar='<start_partition_id>',
            help=_("Name of the partition to start the partition list "
                   "with. The returned partition list does not contain "
                   "this partition."),
        )
        parser.add_argument(
            '--checkpoint-type',
            metavar='<checkpoint_type>',
            default='LAST_READ',
            help=_("Type of the checkpoint.\nLAST_READ: Only "
                   "sequence numbers are recorded in databases."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis
        args_list = (
            'limit',
            'start_partition_id',
            'checkpoint_type',
        )
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        data = client.app_consumptions(parsed_args.streamName,
                                       parsed_args.appName,
                                       **attrs)

        return (
            self.display_columns,
            (
                utils.get_item_properties(
                    s, self.columns, formatters=_formatters
                ) for s in data
            )
        )


class ShowApp(command.ShowOne):
    _description = _("Querying App Details.")

    def get_parser(self, prog_name):
        parser = super(ShowApp, self).get_parser(prog_name)
        parser.add_argument(
            'appName',
            metavar='<appName>',
            help=_("Name of the app to be queried."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis
        obj = client.get_app(parsed_args.appName)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class CreateApp(command.ShowOne):
    _description = _("Create new Consumption App.")

    def get_parser(self, prog_name):
        parser = super(CreateApp, self).get_parser(prog_name)
        parser.add_argument(
            'appName',
            metavar='<appName>',
            help=_("Specifies the name of the App."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis

        attrs = {
            'app_name': parsed_args.appName
        }

        obj = client.create_app(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeleteApp(command.Command):

    _description = _("Delete Consumption App(s).")

    def get_parser(self, prog_name):
        parser = super(DeleteApp, self).get_parser(prog_name)
        parser.add_argument(
            'appName',
            metavar='<appName>',
            nargs='+',
            help=_("Name of Dis App(s) to delete."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis
        result = 0
        for app_name in parsed_args.appName:
            try:
                client.delete_app(app_name)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete App with "
                          "name '%(app_name)s': %(e)s"),
                          {'app_name': app_name, 'e': e})
        if result > 0:
            total = len(parsed_args.appName)
            msg = (_("%(result)s of %(total)s DIS App(s) failed "
                   "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy

from otcextensions.sdk.lts.v2 import group as _group
from otcextensions.sdk.lts.v2 import stream as _stream


class Proxy(proxy.Proxy):

    skip_discovery = True

    def __init__(self, session, *args, **kwargs):
        super(Proxy, self).__init__(session=session, *args, **kwargs)
        self.additional_headers = {"Accept": "application/json",
                                   "Content-type": "application/json"}

    # ======== Group ========
    def groups(self):
        """Query all log groups of an account.

        :returns: A generator of log group
            :class:`~otcextensions.sdk.lts.v2.group.Group` instances
        """
        return self._list(_group.Group)

    def create_group(self, **attrs):
        """Creating a log group for log retention and query

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.lts.v2.group.Group`,
            comprised of the properties on the Group class.
        :returns: The results of config creation
        :rtype: :class:`~otcextensions.sdk.lts.v2.group.Group`
        """
        return self._create(
            _group.Group,
            **attrs
        )

    def update_group(self, group, **attrs):
        """Update log group attributes

        :param group: The id or an instance of
            :class:`~otcextensions.sdk.lts.v2.group.Group`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.lts.v2.group.Group`

        :rtype: :class:`~otcextensions.sdk.lts.v2.group.Group`
        """
        return self._update(_group.Group, group, **attrs)

    def delete_group(self, group, ignore_missing=True):
        """Delete a single log group.

        :param group: The value can be the ID of a log group
             or a :class:`~otcextensions.sdk.lts.v2.group.Group`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent log group.
        :returns:  None
        """
        self._delete(
            _group.Group, group, ignore_missing=ignore_missing
        )
        return None

    # ======== Stream ========
    def streams(self, log_group):
        """List log stream in log group

        :param log_group: The value can be the ID of a log group
            or a :class:`~otcextensions.sdk.lts.v2.group.Group`
            instance.
        :returns: A generator of log stream
            :class:`~otcextensions.sdk.lts.v2.stream.Stream` instances
        :rtype: :class:`~otcextensions.sdk.lts.v2.stream.Stream`
        """
        log_group = self._get_resource(_group.Group, log_group)
        return self._list(
            _stream.Stream,
            log_group_id=log_group.id
        )

    def create_stream(self, log_group, **query):
        """Create a log stream in log group

        :param log_group: The value can be the ID of a log group
            or a :class:`~otcextensions.sdk.lts.v2.group.Group`
            instance.
        :param dict query: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.lts.v2.stream.Stream`,
            comprised of the properties on the Stream class.
        :returns: The result of stream creation.
        """
        log_group = self._get_resource(_group.Group, log_group)
        stream = log_group.create_stream(self, query=query)
        return stream

    def delete_stream(self, log_group, log_stream, ignore_missing=True):
        """Delete log stream from log group

        :param log_group: The value can be the ID of a log group
            or a :class:`~otcextensions.sdk.lts.v2.group.Group`
            instance.
        :param log_stream: The value can be the ID of a log stream
            or a :class:`~otcextensions.sdk.lts.v2.stream.Stream`
            instance.
        :returns:  None
        """
        log_group = self._get_resource(_group.Group, log_group)
        log_stream = self._get_resource(_stream.Stream, log_stream)
        log_group.delete_stream(self, log_stream_id=log_stream.id,
                                ignore_missing=ignore_missing)
        return None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.rds.v1 import instance

# RDS requires those headers to be present in the request, to native API
# otherwise 404
RDS_HEADERS = {
    'Content-Type': 'application/json',
    'X-Language': 'en-us'
}

# RDS requires those headers to be present in the request, to OS-compat API
# otherwise 404
OS_HEADERS = {
    'Content-Type': 'application/json',
}

# PROJECT_ID = '123'
IDENTIFIER = '37f52707-2fb3-482c-a444-77a70a4eafd6'
EXAMPLE = {
    "status": "ACTIVE",
    "name": "rds-new-channle-read",
    "links": [
        {
            "rel": "self",
            "href": ""
        },
        {
            "rel": "bookmark",
            "href": ""
        }
    ],
    "id": IDENTIFIER,
    "volume": {
        "type": "COMMON",
        "size": 100
    },
    "flavor": {
        "id": "7fbf27c5-07e5-43dc-cf13-ad7a0f1c5d9a",
        "links": [
            {
                "rel": "self",
                "href": ""
            },
            {
                "rel": "bookmark",
                "href": ""
            }
        ]
    },
    "datastore": {
        "type": "PostgreSQL",
        "version": "PostgreSQL-9.5.5"
    },
    "publicEndpoint": "10.11.77.101:8635",
    "dbPort": 8635,
    "region": "eu-de",
    "ip": "192.168.1.29",
    "replica_of": [
        {
            "id": "c42cdd29-9912-4b57-91a8-c37a845566b1",
            "links": [
                {
                    "rel": "self",
                    "href": ""
                },
                {
                    "rel": "bookmark",
                    "href": ""
                }
            ]
        }
    ],
    "hostname": 'test'
}


class TestInstance(base.TestCase):

    def setUp(self):
        super(TestInstance, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        # self.sess.get_project_id = mock.Mock(return_value=PROJECT_ID)
        self.sot = instance.Instance(**EXAMPLE)

    def test_basic(self):
        sot = instance.Instance()
        self.assertEqual('instance', sot.resource_key)
        self.assertEqual('instances', sot.resources_key)
        self.assertEqual('/instances', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = instance.Instance(**EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['hostname'], sot.hostname)
        # self.assertEqual(EXAMPLE['links'], sot.links)
        self.assertEqual(EXAMPLE['volume'], sot.volume)
        self.assertEqual(EXAMPLE['flavor'], sot.flavor)
        self.assertEqual(EXAMPLE['datastore'], sot.datastore)
        self.assertEqual(EXAMPLE['publicEndpoint'], sot.publicEndpoint)
        self.assertEqual(EXAMPLE['region'], sot.region)

    def test_list(self):

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {"instances": [
            {"instance": {
                "status": "ACTIVE",
                "name": "rds-new-channle-read",
                "links": [
                    {
                        "rel": "self",
                        "href": ""
                    },
                    {
                        "rel": "bookmark",
                        "href": ""
                    }
                ],
                "id": "37f52707-2fb3-482c-a444-77a70a4eafd6",
                "volume": {
                    "type": "COMMON",
                    "size": 100
                },
                "flavor": {
                    "id": "7fbf27c5-07e5-43dc-cf13-ad7a0f1c5d9a",
                    "links": [
                        {
                            "rel": "self",
                            "href": ""
                        },
                        {
                            "rel": "bookmark",
                            "href": ""
                        }
                    ]
                },
                "datastore": {
                    "type": "PostgreSQL",
                    "version": "PostgreSQL-9.5.5"
                },
                "publicEndpoint": "10.11.77.101:8635",
                "dbPort": 8635,
                "region": "eu-de",
                "ip": "192.168.1.29",
                "replica_of": [
                    {
                        "id": "c42cdd29-9912-4b57-91a8-c37a845566b1",
                        "links": [
                            {
                                "rel": "self",
                                "href": ""
                            },
                            {
                                "rel": "bookmark",
                                "href": ""
                            }
                        ]
                    }
                ],
                "hostname": 'test'
            }
            }
        ]}

        self.sess.get.return_value = mock_response

        result = list(self.sot.list(self.sess))

        self.sess.get.assert_called_once_with(
            '/instances',
            params={},
        )

        self.assertEqual([instance.Instance(**EXAMPLE)], result)

    def test_get(self):

        sot = instance.Instance.new(id='1234')
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}

        res_json = {"instance": {
            "configurationStatus": "In-Sync",
            "paramsGroupId": "b89db814-6ba1-454f-a9ad-380064ef0c6f",
            "type": "MySQL",
            "subnetid": "0fb5d084-4e5d-463b-8920-fca10e6b4028",
            "role": "master",
            "internalSubnetId": "330a10fd-3962-44c5-b3a1-1d282617a183",
            "group": "1",
            "securegroup": "ca99fcef-502f-495f-b28d-85c9c6f4666e",
            "vpc": "292997f2-3bf7-4d60-86a5-4e9d593bc850",
            "azcode": "eu-de-01",
            "region": None,
            "created": "2017-05-12T02:18:46",
            "updated": "2017-05-12T02:18:46",
            "status": "ACTIVE",
            "name": "rds-MySQL-1-1",
            "publicEndpoint": "10.11.77.101:8635",
            "dbPort": 8635,
            "links": [
                {
                    "rel": "self",
                    "href": ""
                },
                {
                    "rel": "bookmark",
                    "href": ""
                }
            ],
            "id": "e8faac23-8129-4c68-a231-480e46fc5f4f",
            "flavor": {
                "id": "31b2863c-0e15-44fd-a80d-1e83a7aca338",
                "links": [
                    {
                        "rel": "self",
                        "href": ""
                    },
                    {
                        "rel": "bookmark",
                        "href": ""
                    }
                ]
            },
            "volume": {
                "type": "COMMON",
                "size": 210,
                "used": 25.07
            },
            "datastore": {
                "type": "MySQL",
                "version": "MySQL-5.7.17"
            },
            "fault": None,
            "configuration": None,
            "locality": None,
            "replicas": None,
            "dbuser": "root",
            "storageEngine": None,
            "payModel": 0,
            "cluster_id": "fb22f24c-0466-48f2-8275-70af04ef4935"
        }
        }

        # Sadly res_json is deleted somewhere in __GET__, so
        # pass a copy of it
        mock_response.json.return_value = copy.deepcopy(res_json)

        self.sess.get.return_value = mock_response

        res = sot.get(self.sess)

        self.sess.get.assert_called_once_with(
            'instances/1234',
        )

        self.assertEqual(res_json['instance']['vpc'], res.vpc)
        self.assertEqual(res_json['instance']['id'], res.id)
        self.assertEqual(res_json['instance']['paramsGroupId'],
                         res.paramsGroupId)

    def test_action_restart(self):
        sot = instance.Instance(**EXAMPLE)
        response = mock.Mock()
        response.json = mock.Mock(return_value='')
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=response)

        self.assertIsNotNone(sot.restart(sess))

        url = ("instances/%(id)s/action" % {
            'id': IDENTIFIER,
        })
        body = {'restart': {}}
        sess.post.assert_called_with(url,
                                     json=body,
                                     headers={'X-Language': 'en-us'})

    def test_action_resize(self):
        sot = instance.Instance(**EXAMPLE)
        response = mock.Mock()
        response.json = mock.Mock(return_value='')
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=response)
        flavor = 'http://flavor/flav'

        self.assertIsNotNone(sot.resize(sess, flavor))

        url = ("instances/%(id)s/action" % {
            'id': IDENTIFIER,
        })
        body = {'resize': {'flavorRef': flavor}}
        sess.post.assert_called_with(url,
                                     json=body,
                                     headers={'X-Language': 'en-us'})

    def test_action_resize_volume(self):
        sot = instance.Instance(**EXAMPLE)
        response = mock.Mock()
        response.json = mock.Mock(return_value='')
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=response)
        size = 4

        self.assertIsNotNone(sot.resize_volume(sess, size))

        url = ("instances/%(id)s/action" % {
            'id': IDENTIFIER,
        })
        body = {'resize': {'volume': size}}
        sess.post.assert_called_with(url,
                                     json=body,
                                     headers={'X-Language': 'en-us'})

    def test_action_restore(self):
        sot = instance.Instance(**EXAMPLE)
        response = mock.Mock()
        response.json = mock.Mock(return_value='')
        sess = mock.Mock()
        sess.post = mock.Mock(return_value=response)
        backupRef = 'backupRef'

        self.assertIsNotNone(sot.restore(sess, backupRef))

        url = ("instances/%(id)s/action" % {
            'id': IDENTIFIER,
        })
        body = {'restore': {'backupRef': backupRef}}
        sess.post.assert_called_with(url,
                                     json=body,
                                     headers={'X-Language': 'en-us'})



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock
# from osc_lib import utils as common_utils

from otcextensions.osclient.rds.v1 import datastore
from otcextensions.tests.unit.osclient.rds.v1 import fakes as rds_fakes


class TestListDatastores(rds_fakes.TestRds):

    columns_datastore_type = ['Name', ]

    def setUp(self):
        super(TestListDatastores, self).setUp()

        self.cmd = datastore.ListDatastores(self.app, None)

        self.app.client_manager.rds.datastore_types = mock.Mock()

        self.datastore_types = []
        self.datastore_type_data = []
        for ds in ['MySQL', 'PostgreSQL', 'SQLServer']:
            obj = type('obj', (object,), {'name': ds})
            self.datastore_types.append(obj)
            # Since only one field is there leave a comma at the end
            self.datastore_type_data.append((ds,))

    def test_list_datastore_types(self):
        arglist = [
        ]

        verifylist = [
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.datastore_types.side_effect = [
            self.datastore_types
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.datastore_types.assert_called()

        self.assertEqual(self.columns_datastore_type, columns)
        self.assertEqual(tuple(self.datastore_type_data), tuple(data))


class TestListDatastoreVersions(rds_fakes.TestRds):
    column_headers = [
        'ID',
        'Name',
        # 'Datastore',
        # 'Image',
        # 'Packages'
    ]

    def setUp(self):
        super(TestListDatastoreVersions, self).setUp()

        self.cmd = datastore.ListDatastoreVersions(self.app, None)

        self.app.client_manager.rds.datastores = mock.Mock()

        self.datastores = self.datastore_mock.create_multiple(3)
        self.datastore_data = []

        for s in self.datastores:
            self.datastore_data.append((
                s.id,
                s.name,
                # s.datastore,
                # s.image,
                # s.packages,
            ))

    def test_list_datastore_versions(self):
        arglist = [
            'test_type'
        ]

        verifylist = [
            ('datastore', 'test_type')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.datastore_versions.side_effect = [
            self.datastores
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.datastore_versions.assert_called()

        self.assertEqual(self.column_headers, columns)
        self.assertEqual(tuple(self.datastore_data), tuple(data))


class TestShowDatastoreVersion(rds_fakes.TestRds):
    columns = ['Active', 'Datastore', 'ID', 'Image', 'Name', 'Packages', ]

    def setUp(self):
        super(TestShowDatastoreVersion, self).setUp()

        self.cmd = datastore.ShowDatastoreVersion(self.app, None)

        self.app.client_manager.rds.datastores = mock.Mock()

        self.datastore = self.datastore_mock.create_one()
        # self.datastore_data = []

        self.datastore_data = (
            self.datastore.active,
            self.datastore.datastore,
            self.datastore.id,
            self.datastore.image,
            self.datastore.name,
            self.datastore.packages,
        )

    def test_show_datastore_versions(self):
        arglist = [
            self.datastore.id,
            '--datastore=test_ds'
        ]

        verifylist = [
            ('datastore', 'test_ds'),
            ('datastore_version', self.datastore.id)
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.get_datastore_version.side_effect = [
            self.datastore
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.get_datastore_version.assert_called()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datastore_data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import utils


class AppCSpec(resource.Resource):
    app_consistency = resource.Body('app_consistency')
    error_code = resource.Body('app_consistency_error_code')
    error_message = resource.Body('app_consistency_error_message')
    error_status = resource.Body('app_consistency_error_status')


class ExtendInfo(resource.Resource):
    app_consistency = resource.Body('app_consistency', type=AppCSpec)
    architecture = resource.Body('architecture')
    #: Whether the backup is automatically triggered
    auto_trigger = resource.Body('auto_trigger', type=bool)
    #: Whether the backup is a system disk backup
    bootable = resource.Body('bootable', type=bool)
    #: Whether the VM backup data contains system disk data
    contain_system_disk = resource.Body('contain_system_disk', type=bool)
    #: Whether the backup is encrypted
    encrypted = resource.Body('encrypted', type=bool)
    #: Whether the backup is an incremental backup
    incremental = resource.Body('incremental', type=bool)
    #: ID list of images created using backups
    os_images_data = resource.Body('os_images_data')
    progress = resource.Body('progress')
    #: Snapshot ID of the disk backup
    snapshot_id = resource.Body('snapshot_id')
    #: Whether to allow lazyloading for fast restoration
    support_lld = resource.Body('support_lld', type=bool)
    #: Restoration mode
    #: Choices: na, snapshot, backup
    #: Default: na
    #: na indicates the backup cannot be used for restoration
    supported_restore_mode = resource.Body('supported_restore_mode')
    #: Whether the disk is a system disk
    system_disk = resource.Body('system_disk', type=bool)


class Backup(resource.Resource):
    """CBR Backup Resource"""
    resource_key = 'backup'
    resources_key = 'backups'
    base_path = '/backups'

    # capabilities
    allow_create = False
    allow_list = True
    allow_fetch = True
    allow_delete = True
    allow_commit = False

    _query_mapping = resource.QueryParameters(
        'checkpoint_id', 'dec', 'end_time', 'image_type', 'limit',
        'marker', 'member_status', 'name', 'offset', 'own_type',
        'parent_id', 'resource_az', 'resource_id', 'resource_name',
        'resource_type', 'sort', 'start_time', 'status',
        'used_percent', 'vault_id')

    #: Properties
    #: Restore point ID
    checkpoint_id = resource.Body('checkpoint_id')
    children = resource.Body('children', type=list)
    #: Creation time
    created_at = resource.Body('created_at')
    #: Backup description
    description = resource.Body('description')
    #: Expiration time
    expired_at = resource.Body('expired_at')
    #: Extended Information
    extend_info = resource.Body('extend_info', type=ExtendInfo)
    #: Backup type
    #: Values: backup or replication
    image_type = resource.Body('image_type')
    #: Backup name
    name = resource.Body('name')
    #: Parent backup ID
    parent_id = resource.Body('parent_id')
    #: Project ID
    project_id = resource.Body('project_id')
    #: Backup time
    protected_at = resource.Body('protected_at')
    provider_id = resource.Body('provider_id')
    replication_records = resource.Body('replication_records', type=list)
    #: Resource availability zone
    resource_az = resource.Body('resource_az')
    #: Resource ID
    resource_id = resource.Body('resource_id')
    #: Resource name
    resource_name = resource.Body('resource_name')
    #: Resource type:
    #: OS::Nova::Server or OS::Cinder::Volume
    resource_type = resource.Body('resource_type')
    #: Resource size in GB
    resource_size = resource.Body('resource_size', type=int)
    #: Backup status
    status = resource.Body('status')
    #: Update time
    updated_at = resource.Body('updated_at')
    #: Vault ID
    vault_id = resource.Body('vault_id')

    def add_members(self, session, members):
        """Method to add several share members to a backup

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param list members: List of target project IDs to which the backup
            is shared
        """
        url = utils.urljoin(self.base_path, self.id, '/members')
        body = {
            'members': members
        }
        return session.post(url, json=body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy
import json
import os

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.volume_backup.v2 import backup_task as _backup_task

TESTDATA_DIR = os.path.join(os.path.dirname(__file__))


def _get_fixture(name):
    fixture = os.path.join(
        TESTDATA_DIR,
        'data_files/',
        name)
    with open(fixture, 'r') as data:
        return json.load(data)


EXAMPLE = {
    "status": "EXECUTE_SUCCESS",
    "job_id": "c11b5a18-4559-4731-b7b3-58e2bd89cdb9",
    "created_at": "2016-12-02T09:06:46.706",
    "finished_at": "2016-12-02T13:00:00.121",
    "backup_name": "autobk_e6d2",
    "resource_id": "f47a4ab5-11f5-4509-97f5-80ce0dd74e37",
    "resource_type": "volume"
}


class TestBackupPolicyTask(base.TestCase):

    def setUp(self):
        super(TestBackupPolicyTask, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()

        self.sot = _backup_task.BackupTask()

    def test_basic(self):
        sot = self.sot
        self.assertEqual('tasks', sot.resources_key)
        self.assertEqual(
            '/backuppolicy/%(policy_id)s/backuptasks', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_update)

    def test_make_it(self):
        data = EXAMPLE
        sot = _backup_task.BackupTask.existing(**data)
        self.assertEqual(data['job_id'], sot.id)
        self.assertEqual(data['backup_name'], sot.backup_name)
        self.assertEqual(data['status'], sot.status)
        self.assertEqual(data['resource_id'], sot.resource_id)
        self.assertEqual(data['resource_type'], sot.resource_type)
        if data['created_at']:
            self.assertEqual(data['created_at'], sot.created_at)
        self.assertEqual(data['finished_at'], sot.finished_at)

    def test_list(self):
        response = _get_fixture('list_tasks.json')
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = copy.deepcopy(response)

        self.sess.get.return_value = mock_response

        result = list(self.sot.list(self.sess, policy_id='pol_id'))

        self.sess.get.assert_called_once_with(
            '/backuppolicy/%s/backuptasks' % 'pol_id',
            params={}
        )

        self.assertIsNotNone(result)

        for bp in result:
            assert isinstance(bp, _backup_task.BackupTask)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class AclApiBinding(resource.Resource):
    base_path = f'apigw/instances/%(gateway_id)s/acl-bindings'
    gateway_id = resource.URI('gateway_id', type=str)
    resources_key = 'acl_bindings'
    id = resource.Body('id')
    api_id = resource.Body('api_id')
    env_id = resource.Body('env_id')
    acl_id = resource.Body('acl_id')
    create_time = resource.Body('create_time')
    allow_delete = True

    def _bind_to_api(self, session, gateway_id, **attrs):
        uri = f'apigw/instances/{gateway_id}/acl-bindings'
        response = session.post(uri, json=attrs)
        exceptions.raise_from_response(response)
        data = response.json()
        resources = data[self.resources_key]
        for raw_resource in resources:
            value = self.existing(
                connection=session._get_connection(),
                **raw_resource)
            yield value


class AclBindingFailure(resource.Resource):
    resources_key = 'failure'
    bind_id = resource.Body('bind_id')
    error_code = resource.Body('error_code')
    error_msg = resource.Body('error_msg')
    api_id = resource.Body('api_id')
    api_name = resource.Body('api_name')

    def _unbind_multiple_acls(self, session, gateway_id, **attrs):
        uri = f'apigw/instances/{gateway_id}/acl-bindings?action=DELETE'
        response = session.put(uri, json=attrs)
        exceptions.raise_from_response(response)
        data = response.json()
        resources = data[self.resources_key]
        for raw_resource in resources:
            value = self.existing(
                connection=session._get_connection(),
                **raw_resource)
            yield value


class ApiForAcl(resource.Resource):
    base_path = f'apigw/instances/%(gateway_id)s/acl-bindings/binded-apis'
    gateway_id = resource.URI('gateway_id', type=str)
    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'acl_id', 'api_id', 'api_name',
        'env_id', 'group_id'
    )
    allow_list = True
    resources_key = 'apis'
    api_id = resource.Body('api_id')
    api_name = resource.Body('api_name')
    api_type = resource.Body('api_type')
    api_remark = resource.Body('api_remark')
    env_id = resource.Body('env_id')
    env_name = resource.Body('env_name')
    bind_id = resource.Body('bind_id')
    group_name = resource.Body('group_name')
    bind_time = resource.Body('bind_time')
    publish_id = resource.Body('publish_id')
    req_method = resource.Body('req_method')


class UnbindApiForAcl(resource.Resource):
    base_path = f'apigw/instances/%(gateway_id)s/acl-bindings/unbinded-apis'
    gateway_id = resource.URI('gateway_id', type=str)
    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'acl_id', 'api_id', 'api_name',
        'env_id', 'group_id'
    )
    allow_list = True
    resources_key = 'apis'
    id = resource.Body('id')
    name = resource.Body('name')
    group_id = resource.Body('group_id')
    group_name = resource.Body('group_name')
    type = resource.Body('type', type=int)
    remark = resource.Body('remark')
    run_env_name = resource.Body('run_env_name')
    run_env_id = resource.Body('run_env_id')
    publish_id = resource.Body('publish_id')
    acl_name = resource.Body('acl_name')
    req_uri = resource.Body('req_uri')
    auth_type = resource.Body('auth_type')
    req_method = resource.Body('req_method')


class AclForApi(resource.Resource):
    base_path = f'apigw/instances/%(gateway_id)s/acl-bindings/binded-acls'
    gateway_id = resource.URI('gateway_id', type=str)
    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'acl_id', 'api_id', 'env_name',
        'env_id', 'group_id'
    )
    allow_list = True

    resources_key = 'acls'
    acl_id = resource.Body('acl_id')
    acl_name = resource.Body('acl_name')
    entity_type = resource.Body('entity_type')
    acl_type = resource.Body('acl_type')
    acl_value = resource.Body('acl_value')
    env_id = resource.Body('env_id')
    env_name = resource.Body('env_name')
    bind_id = resource.Body('bind_id')
    bind_time = resource.Body('bind_time')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.tms.v1 import resource_tag

EXAMPLE = {
    "project_id": "786ef11caa5c43ff80256be4c7fee8b7",
    "resources": [{"resource_id": "2079d0a6-3dbc-4d59-99da-6b8b7c899a97",
                   "resource_type": "vpc"}],
    "tags": [{"key": "ENV1", "value": "dev1"}],
}


class TestResourceTag(base.TestCase):
    def test_basic(self):
        sot = resource_tag.ResourceTag()
        path = "/resource-tags"
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = resource_tag.ResourceTag(**EXAMPLE)
        self.assertEqual(EXAMPLE['tags'], sot.tags)
        self.assertEqual(EXAMPLE['resources'], sot.resources)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Configuration v3 action implementations"""

from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils


DATASTORE_TYPE_CHOICES = ['mysql', 'postgresql', 'sqlserver']


_formatters = {
}


def _get_columns(item, skip_values=True):
    column_map = {}
    hidden = ['location', 'links']
    if skip_values:
        if 'configuration_parameters' in item:
            hidden.append('configuration_parameters')
        if 'values' in item:
            hidden.append('values')

    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden)


class ListConfigurations(command.Lister):
    _description = _("List Configurations")
    column_headers = [
        'ID', 'Name', 'Description', 'Datastore Name',
        'Datastore Version Name', 'User Defined'
    ]

    columns = [
        'id', 'name', 'description', 'datastore_name',
        'datastore_version_name', 'is_user_defined'
    ]

    def get_parser(self, prog_name):
        parser = super(ListConfigurations, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        data = client.configurations()

        return (self.column_headers, (utils.get_item_properties(
            s,
            self.columns,
        ) for s in data))


class ShowConfiguration(command.ShowOne):
    _description = _("Show details of a database configuration")

    def get_parser(self, prog_name):
        parser = super(ShowConfiguration, self).get_parser(prog_name)
        parser.add_argument('configuration',
                            metavar="<configuration>",
                            help=_("ID or name of the configuration"))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        obj = client.find_configuration(parsed_args.configuration,
                                        ignore_missing=False)

        display_columns, columns = _get_columns(obj, skip_values=True)
        data = utils.get_item_properties(obj, columns,
                                         formatters=_formatters)

        return (display_columns, data)


class ListConfigurationParameters(command.Lister):
    _description = _("List Configuration parameters")
    column_headers = (
        'Name', 'Value', 'Type', 'Description',
        'Restart Required', 'Readonly', 'Value Range'
    )
    columns = (
        'name', 'value', 'type', 'description',
        'restart_required', 'readonly', 'value_range'
    )

    def get_parser(self, prog_name):
        parser = super(ListConfigurationParameters, self).get_parser(prog_name)
        parser.add_argument('configuration',
                            metavar="<configuration>",
                            help=_("ID or name of the configuration"))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        data = client.find_configuration(
            parsed_args.configuration,
            ignore_missing=False).configuration_parameters

        return (self.column_headers, (utils.get_dict_properties(
            s,
            self.columns,
        ) for s in data))


class CreateConfiguration(command.ShowOne):
    _description = _("Create new Configuration")

    def get_parser(self, prog_name):
        parser = super(CreateConfiguration, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar="<name>",
            help=_("Parameter group name"))
        parser.add_argument(
            '--description',
            metavar="<description>",
            help=_("Parameter group description"))
        parser.add_argument(
            '--datastore-type',
            metavar='{' + ','.join(DATASTORE_TYPE_CHOICES) + '}',
            choices=DATASTORE_TYPE_CHOICES,
            type=lambda s: s.lower(),
            required=True,
            help=_("Datastore type"))
        parser.add_argument(
            '--datastore-version',
            metavar="<datastore_version>",
            required=True,
            help=_("Datastore version"))
        parser.add_argument(
            '--value',
            dest='values',
            metavar="<key=value>",
            action=parseractions.KeyValueAction,
            help=_("Configuration value"
                   "(repeat option to set multiple values)")
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        config_attrs = {}
        config_attrs['datastore'] = {}
        if parsed_args.name:
            config_attrs['name'] = parsed_args.name
        if parsed_args.description:
            config_attrs['description'] = parsed_args.description
        if parsed_args.datastore_type and parsed_args.datastore_version:
            config_attrs['datastore'] = {
                'type': parsed_args.datastore_type,
                'version': parsed_args.datastore_version
            }

        if parsed_args.values:
            config_attrs['values'] = parsed_args.values
        obj = client.create_configuration(**config_attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class ApplyConfiguration(command.Lister):
    _description = _("Apply Configuration to the instance(s)")

    columns = (
        'instance_id', 'instance_name',
        'restart_required', 'success'
    )

    column_headers = (
        'ID', 'Name', 'Restart required', 'success'
    )

    def get_parser(self, prog_name):
        parser = super(ApplyConfiguration, self).get_parser(prog_name)
        parser.add_argument(
            'configuration',
            metavar="<configuration>",
            help=_("Configuration name or id")
        )
        parser.add_argument(
            '--instance',
            metavar="<instance_id>",
            dest='instances',
            action='append',
            required=True,
            help=_('ID of the instance the configuration '
                   'should be applied to. '
                   '(repeat option to apply to multiple instances).')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        config = client.find_configuration(parsed_args.configuration,
                                           ignore_missing=False)

        inst_ids = []

        # Ensure instance_ids are right
        for inst in parsed_args.instances:
            inst_ids.append(client.get_instance(inst).id)

        obj = client.apply_configuration(config.id, instances=inst_ids)

        return (self.column_headers, (utils.get_dict_properties(
            s,
            self.columns,
        ) for s in obj.apply_results))


class SetConfiguration(command.Command):
    _description = _("Set values of the Configuration")

    columns = ('id', 'name', 'description', 'datastore_version_id',
               'datastore_version_name', 'datastore_name', 'created',
               'updated', 'allowed_updated', 'instance_count', 'values')

    def get_parser(self, prog_name):
        parser = super(SetConfiguration, self).get_parser(prog_name)
        parser.add_argument('configuration',
                            metavar="<configuration>",
                            help=_("Configuration id"))
        parser.add_argument('--name',
                            metavar="<name>",
                            help=_("New Configuration name"))
        parser.add_argument('--description',
                            metavar="<description>",
                            help=_("New Configuration description"))
        parser.add_argument(
            '--value',
            dest="values",
            metavar="<key=value>",
            required=True,
            action=parseractions.KeyValueAction,
            help=_("Configuration value"
                   "(repeat option to set multiple values)."))

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        config_attrs = {}

        if parsed_args.name:
            config_attrs['name'] = parsed_args.name
        if parsed_args.description:
            config_attrs['description'] = parsed_args.description
        if parsed_args.values:
            config_attrs['values'] = parsed_args.values

        config = client.find_configuration(parsed_args.configuration,
                                           ignore_missing=False)

        client.update_configuration(config, **config_attrs)


class DeleteConfiguration(command.Command):
    _description = _("Delete a configuration")

    def get_parser(self, prog_name):
        parser = super(DeleteConfiguration, self).get_parser(prog_name)
        parser.add_argument('configuration',
                            metavar='<configuration>',
                            nargs='+',
                            help=_('ID or name of the configuration group'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds
        if parsed_args.configuration:
            for cnf in parsed_args.configuration:
                resource = client.find_configuration(cnf, ignore_missing=False)
                client.delete_configuration(resource)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Message Publishing v2 action implementations"""

import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class SendSms(command.ShowOne):
    _description = _("Send a transactional SMS message to a specified "
                     "phone number, usually used for verification code "
                     "or notification.")

    def get_parser(self, prog_name):
        parser = super(SendSms, self).get_parser(prog_name)
        parser.add_argument(
            'endpoint',
            metavar='<endpoint>',
            help=_("Phone number."),
        )
        parser.add_argument(
            'message',
            metavar='<message>',
            help=_("SMS message content."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.smn

        args_list = [
            'endpoint',
            'message']
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        obj = client.send_sms(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.sdk.dcaas.v2 import _proxy
from otcextensions.sdk.dcaas.v2 import endpoint_group as _endpoint_group

from openstack.tests.unit import test_proxy_base


class TestDcaasProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestDcaasProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_endpoint_groups(self):
        self.verify_list(
            self.proxy.endpoint_groups,
            _endpoint_group.DirectConnectEndpointGroup
        )

    def test_endpoint_groups_query(self):
        self.verify_list(
            self.proxy.endpoint_groups,
            _endpoint_group.DirectConnectEndpointGroup,
            method_kwargs={
                'name': 'test_name',
                'description': 'test description',
                'endpoints': ['10.2.0.0/24', '10.3.0.0/24'],
                'tenant_id': '6fbe9263116a4b68818cf1edce16bc4f',
                'type': 'cidr'
            },
            expected_kwargs={
                'name': 'test_name',
                'description': 'test description',
                'endpoints': ['10.2.0.0/24', '10.3.0.0/24'],
                'tenant_id': '6fbe9263116a4b68818cf1edce16bc4f',
                'type': 'cidr'
            }
        )

    def test_create_endpoint_group(self):
        self.verify_create(
            self.proxy.create_endpoint_group,
            _endpoint_group.DirectConnectEndpointGroup,
            method_kwargs={
                'name': 'test_name',
                'tenant_id': '6fbe9263116a4b68818cf1edce16bc4f',
                'endpoints': ['10.2.0.0/24', '10.3.0.0/24'],
                'type': 'cidr'
            },
            expected_kwargs={
                'prepend_key': False,
                'name': 'test_name',
                'tenant_id': '6fbe9263116a4b68818cf1edce16bc4f',
                'endpoints': ['10.2.0.0/24', '10.3.0.0/24'],
                'type': 'cidr'
            }
        )

    def test_get_endpoint_group(self):
        self.verify_get(self.proxy.get_endpoint_group,
                        _endpoint_group.DirectConnectEndpointGroup
                        )

    def test_find_endpoint_group(self):
        self.verify_find(self.proxy.find_endpoint_group,
                         _endpoint_group.DirectConnectEndpointGroup,
                         method_kwargs={'id': 'test_id'},
                         expected_kwargs={'id': 'test_id'}
                         )

    def test_update_endpoint_group(self):
        self.verify_update(self.proxy.update_endpoint_group,
                           _endpoint_group.DirectConnectEndpointGroup,
                           method_kwargs={'id': 'test_id'},
                           expected_kwargs={'id': 'test_id'}
                           )

    def test_delete_endpoint_group(self):
        self.verify_delete(self.proxy.delete_endpoint_group,
                           _endpoint_group.DirectConnectEndpointGroup
                           )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.cbr.v3 import member
from otcextensions.tests.unit.osclient.cbr.v3 import fakes


class TestMember(fakes.TestCBR):

    def setUp(self):
        super(TestMember, self).setUp()

    def test_flatten(self):
        obj = fakes.FakeMember.create_one()

        flat_data = member._flatten_member(obj)

        data = (
            flat_data['id'],
            flat_data['status'],
            flat_data['created_at'],
            flat_data['updated_at'],
            flat_data['backup_id'],
            flat_data['image_id'],
            flat_data['dest_project_id'],
            flat_data['vault_id']
        )

        cmp_data = (
            obj.id,
            obj.status,
            obj.created_at,
            obj.updated_at,
            obj.backup_id,
            obj.image_id,
            obj.dest_project_id,
            obj.vault_id
        )

        self.assertEqual(data, cmp_data)


class TestListMembers(fakes.TestCBR):

    objects = fakes.FakeMember.create_multiple(3)
    backup_obj = fakes.FakeBackup.create_one()

    columns = ('id', 'status', 'created_at', 'updated_at', 'backup_id',
               'image_id', 'dest_project_id', 'vault_id')

    data = []

    for s in objects:
        flat_data = member._flatten_member(s)
        data.append((
            flat_data['id'],
            flat_data['status'],
            flat_data['created_at'],
            flat_data['updated_at'],
            flat_data['backup_id'],
            flat_data['image_id'],
            flat_data['dest_project_id'],
            flat_data['vault_id']
        ))

    def setUp(self):
        super(TestListMembers, self).setUp()

        self.cmd = member.ListMembers(self.app, None)

        self.client.members = mock.Mock()
        self.client.find_backup = mock.Mock()
        self.client.api_mock = self.client.members

    def test_default(self):
        arglist = [
            'backup',
            '--dest-project-id', 'project_uuid',
            '--image-id', 'image_uuid',
            '--limit', '5',
            '--marker', 'marker',
            '--offset', '5',
            '--sort', 'sort',
            '--status', 'status',
            '--vault-id', 'vault_uuid'
        ]

        verifylist = [
            ('dest_project_id', 'project_uuid'),
            ('image_id', 'image_uuid'),
            ('limit', 5),
            ('marker', 'marker'),
            ('offset', 5),
            ('sort', 'sort'),
            ('status', 'status'),
            ('vault_id', 'vault_uuid'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]
        self.client.find_backup.side_effect = [
            self.backup_obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            backup=self.backup_obj.id,
            dest_project_id='project_uuid',
            image_id='image_uuid',
            limit=5,
            marker='marker',
            offset=5,
            sort='sort',
            status='status',
            vault_id='vault_uuid'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowMember(fakes.TestCBR):

    object = fakes.FakeMember.create_one()
    backup_obj = fakes.FakeBackup.create_one()

    columns = ('id', 'status', 'created_at', 'updated_at', 'backup_id',
               'image_id', 'dest_project_id', 'vault_id')

    flat_data = member._flatten_member(object)

    data = (
        flat_data['id'],
        flat_data['status'],
        flat_data['created_at'],
        flat_data['updated_at'],
        flat_data['backup_id'],
        flat_data['image_id'],
        flat_data['dest_project_id'],
        flat_data['vault_id']
    )

    def setUp(self):
        super(TestShowMember, self).setUp()

        self.cmd = member.ShowMember(self.app, None)

        self.client.get_member = mock.Mock()
        self.client.find_backup = mock.Mock()

    def test_default(self):
        arglist = [
            'backup',
            'member'
        ]
        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_backup.side_effect = [
            self.backup_obj
        ]
        self.client.get_member.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.get_member.assert_called_once_with(
            member='member',
            backup=self.backup_obj.id,
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestAddMembers(fakes.TestCBR):

    object = fakes.FakeMember.create_one()
    backup_obj = fakes.FakeBackup.create_one()

    columns = ('id', 'status', 'created_at', 'updated_at', 'backup_id',
               'image_id', 'dest_project_id', 'vault_id')

    data = []
    flat_data = member._flatten_member(object)
    data.append((
        flat_data['id'],
        flat_data['status'],
        flat_data['created_at'],
        flat_data['updated_at'],
        flat_data['backup_id'],
        flat_data['image_id'],
        flat_data['dest_project_id'],
        flat_data['vault_id']
    ))

    def setUp(self):
        super(TestAddMembers, self).setUp()

        self.cmd = member.AddMembers(self.app, None)
        self.app.client_manager.sdk_connection = mock.Mock()

        self.backup_client = self.app.client_manager.cbr
        self.backup_client.add_members = mock.Mock()
        self.client.find_backup = mock.Mock()

    def test_default(self):
        arglist = [
            'backup',
            '--members', 'project_uuid'
        ]
        verifylist = [
            ('backup', 'backup'),
            ('members', ['project_uuid'])
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.backup_client.add_members.side_effect = [
            [self.object]
        ]

        self.client.find_backup.side_effect = [
            self.backup_obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.backup_client.add_members.assert_called_once_with(
            backup=self.backup_obj.id,
            members=['project_uuid']
        )

        self.assertEqual(self.columns, columns)
        data_list = []
        for d in data:
            data_list.append(d)
        self.assertEqual(self.data, data_list)


class TestUpdateMember(fakes.TestCBR):

    object = fakes.FakeMember.create_one()

    columns = ('id', 'status', 'created_at', 'updated_at', 'backup_id',
               'image_id', 'dest_project_id', 'vault_id')

    flat_data = member._flatten_member(object)

    data = (
        flat_data['id'],
        flat_data['status'],
        flat_data['created_at'],
        flat_data['updated_at'],
        flat_data['backup_id'],
        flat_data['image_id'],
        flat_data['dest_project_id'],
        flat_data['vault_id']
    )

    def setUp(self):
        super(TestUpdateMember, self).setUp()

        self.cmd = member.UpdateMember(self.app, None)
        self.app.client_manager.sdk_connection = mock.Mock()

        self.client.update_member = mock.Mock()

    def test_default(self):
        arglist = [
            'backup',
            'member',
            '--status', 'pending',
            '--vault-id', 'vault_uuid'
        ]
        verifylist = [
            ('backup', 'backup'),
            ('member', 'member'),
            ('status', 'pending'),
            ('vault_id', 'vault_uuid')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_member.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_member.assert_called_once_with(
            backup='backup',
            member='member',
            status='pending',
            vault='vault_uuid'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteMember(fakes.TestCBR):

    backup_obj = fakes.FakeBackup.create_one()

    def setUp(self):
        super(TestDeleteMember, self).setUp()

        self.cmd = member.DeleteMember(self.app, None)

        self.client.delete_member = mock.Mock()

    def test_delete(self):
        arglist = [
            'backup',
            'member-uuid'
        ]
        verifylist = [
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_member.side_effect = [{}]

        # Set the response for find_policy
        self.client.find_backup.side_effect = [
            self.backup_obj
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        delete_calls = [
            mock.call(
                backup=self.backup_obj.id,
                member='member-uuid',
                ignore_missing=False),
        ]

        self.client.delete_member.assert_has_calls(delete_calls)
        self.assertEqual(1, self.client.delete_member.call_count)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from otcextensions.tests.functional.sdk.elb import TestElb


class TestIamCustomRoles(TestElb):

    def setUp(self):
        super(TestIamCustomRoles, self).setUp()
        self.client = self.conn.identity

    def test_list_roles(self):
        query = {}
        roles = list(self.client.custom_roles(**query))
        self.assertGreaterEqual(len(roles), 0)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy
from openstack import resource
from otcextensions.sdk.cbr.v3 import backup as _backup
from otcextensions.sdk.cbr.v3 import checkpoint as _checkpoint
from otcextensions.sdk.cbr.v3 import member as _member
from otcextensions.sdk.cbr.v3 import policy as _policy
from otcextensions.sdk.cbr.v3 import restore as _restore
from otcextensions.sdk.cbr.v3 import task as _task
from otcextensions.sdk.cbr.v3 import vault as _vault


class Proxy(proxy.Proxy):

    skip_discovery = True

    # ======== Backup ========
    def backups(self, **query):
        """Retrieve a generator of Backups

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * `checkpoint_id`: Restore point ID
            * `dec`: Dedicated Cloud
            * `end_time`: Time when the backup ends
            * `image_type`: Backup type: backup or replication
            * `limit`: Number of records displayed per page
            * `marker`: ID of the last record displayed
            * `member_status`: Backup sharing status
            * `name`: Backup name
            * `offset`: Offset value
            * `parent_id`: Parent backup ID
            * `resource_az`: AZ-based filtering
            * `resource_id`: Resource ID
            * `resource_name`: Resource name
            * `resource_type`: Resource Type
            * `sort`: sorting
            * `start_time`: Time when the backup starts
            * `status`: Status
            * `vault_id`: Vault ID

        :returns: A generator of backup
            :class:`~otcextensions.sdk.cbr.v3.backup.Backup` instances
        """
        return self._list(_backup.Backup, **query)

    def get_backup(self, backup):
        """Get the backup by UUID.

        :param backup: key id or an instance of
            :class:`~otcextensions.sdk.cbr.v3.backup.Backup`

        :returns: instance of
            :class:`~otcextensions.sdk.cbr.v3.backup.Backup`
        """
        return self._get(
            _backup.Backup, backup
        )

    def find_backup(self, name_or_id, ignore_missing=True):
        """Find a single CBR backup by name or id

        :param name_or_id: The name or ID of a backup
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the group does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent group.

        :returns: ``None``
        """
        return self._find(
            _backup.Backup, name_or_id,
            ignore_missing=ignore_missing,
        )

    def delete_backup(self, backup, ignore_missing=True):
        """Delete a single CBR backup.

        :param backup: The value can be the ID of a backup
             or a :class:`~otcextensions.sdk.cbr.v3.backup.Backup`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent backup.
        """
        return self._delete(
            _backup.Backup, backup, ignore_missing=ignore_missing,
        )

    # ======== Checkpoint / Restore Point ========
    def get_checkpoint(self, checkpoint):

        """Get the checkpoint by UUID.

        :param checkpoint: key id or an instance of
            :class:`~otcextensions.sdk.cbr.v3.checkpoint.Checkpoint`

        :returns: instance of
            :class:`~otcextensions.sdk.cbr.v3.checkpoint.Checkpoint`
        """
        return self._get(
            _checkpoint.Checkpoint, checkpoint
        )

    def create_checkpoint(self, **attrs):
        """Creating a restore point / checkpoint using attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.cbr.v3.checkpoint.Checkpoint`,
            comprised of the properties on the Checkpoint class.
        :returns: The results of config creation
        :rtype: :class:`~otcextensions.sdk.cbr.v3.checkpoint.Checkpoint`
        """
        return self._create(
            _checkpoint.Checkpoint,
            **attrs
        )

    def wait_for_checkpoint(self, checkpoint, status='available',
                            failures=None,
                            interval=2, wait=300, attribute='status'):
        """Wait for an checkpoint to be in a particular status.

        :param checkpoint:
            The :class:`~otcextensions.sdk.cbr.v3.checkpoint.Checkpoint`
            or checkpoint ID to wait on to reach the specified status.
        :param status: Desired status.
        :param failures:
            Statuses that would be interpreted as failures.
        :type failures: :py:class:`list`
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait before the change.
            Default to 180
        :return: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
                 to the desired status failed to occur in specified seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
                 has transited to one of the failure statuses.
        """
        failures = ['error'] if failures is None else failures
        return resource.wait_for_status(
            self, checkpoint, status, failures, interval, wait)

    # ======== Policy ========
    def policies(self, **query):

        """Retrieve a generator of CBR policies

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * `operation_type`: Policy type: backup or replication
            * `vault_id`: Vault ID

        :returns: A generator of policies
            :class:`~otcextensions.sdk.cbr.v3.policy.Policy` instances
        """
        return self._list(_policy.Policy, **query)

    def get_policy(self, policy):
        """Get the CBR policy by UUID.

        :param policy: key id or an instance of
            :class:`~otcextensions.sdk.cbr.v3.policy.Policy`

        :returns: instance of
            :class:`~otcextensions.sdk.cbr.v3.policy.Policy`
        """
        return self._get(
            _policy.Policy, policy
        )

    def find_policy(self, name_or_id, ignore_missing=True):
        """Find a single CBR policy by name or ID

        :param name_or_id: The name or ID of a policy
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the policy does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent policy.

        :returns: a :class:`~otcextensions.sdk.cbr.v3.policy.Policy` instance
        """
        return self._find(_policy.Policy, name_or_id,
                          ignore_missing=ignore_missing)

    def create_policy(self, **attrs):
        """Creating a CBR policy using attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.cbr.v3.policy.Policy`,
            comprised of the properties on the Policy class.
        :returns: The results of config creation
        :rtype: :class:`~otcextensions.sdk.cbr.v3.policy.Policy`
        """
        return self._create(
            _policy.Policy,
            **attrs
        )

    def update_policy(self, policy, **attrs):
        """Update CBR policy attributes

        :param policy: The id or an instance of
            :class:`~otcextensions.sdk.cbr.v3.policy.Policy`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.cbr.v3.policy.Policy`

        :rtype: :class:`~otcextensions.sdk.cbr.v3.policy.Policy`
        """
        return self._update(_policy.Policy, policy, **attrs)

    def delete_policy(self, policy, ignore_missing=True):
        """Delete a single CBR policy.

        :param policy: The value can be the ID of a policy
             or a :class:`~otcextensions.sdk.cbr.v3.policy.Policy`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent policy.
        """
        return self._delete(
            _policy.Policy, policy, ignore_missing=ignore_missing,
        )

    # ======== Restore ========
    def restore_data(self, backup, **attrs):
        """Restore data using a backup

        :param backup: The value can be the ID of a backup
             or a :class:`~otcextensions.sdk.cbr.v3.backup.Backup`
             instance.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.cbr.v3.restore.Restore`,
            comprised of the properties on the Restore class.
        :returns: The results of config creation
        :rtype: :class:`~otcextensions.sdk.cbr.v3.restore.Restore`
        """
        backup = self._get_resource(_backup.Backup, backup)
        return self._create(
            _restore.Restore,
            backup_id=backup.id,
            **attrs
        )

    # ======== Vault ========
    def vaults(self, **query):
        """Retrieve a generator of Vaults

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * `cloud_type`: Cloud type
            * `id`: Vault ID
            * `limit`: Number of records displayed per page, range: 1-100
            * `name`: Vault name
            * `object_type`: Resource type
            * `offset`: Offset value
            * `policy_id`: Policy ID
            * `protect_type`: Protection type
            * `resource_ids`: Resource IDs
            * `status`: Status

        :returns: A generator of vault
            :class:`~otcextensions.sdk.cbr.v3.vault.Vault` instances
        """
        return self._list(_vault.Vault, **query)

    def get_vault(self, vault):
        """Get the vault by UUID.

        :param vault: key id or an instance of
            :class:`~otcextensions.sdk.cbr.v3.vault.Vault`

        :returns: instance of
            :class:`~otcextensions.sdk.cbr.v3.vault.Vault`
        """
        return self._get(
            _vault.Vault, vault
        )

    def find_vault(self, name_or_id, ignore_missing=True):
        """Find a single CBR vault by name or id

        :param name_or_id: The name or ID of a vault
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the group does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent vault.

        :returns: ``None``
        """
        return self._find(
            _vault.Vault, name_or_id,
            ignore_missing=ignore_missing,
        )

    def create_vault(self, **attrs):
        """Creating a CBR vault using attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.cbr.v3.vault.Vault`,
            comprised of the properties on the Vault class.
        :returns: The results of config creation
        :rtype: :class:`~otcextensions.sdk.cbr.v3.vault.Vault`
        """
        return self._create(
            _vault.Vault,
            **attrs
        )

    def update_vault(self, vault, **attrs):
        """Update CBR vault attributes

        :param vault: The id or an instance of
            :class:`~otcextensions.sdk.cbr.v3.vault.Vault`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.cbr.v3.vault.Vault`

        :rtype: :class:`~otcextensions.sdk.cbr.v3.vault.Vault`
        """
        return self._update(_vault.Vault, vault, **attrs)

    def delete_vault(self, vault, ignore_missing=True):
        """Delete a single CBR vault.

        :param vault: The value can be the ID of a vault
             or a :class:`~otcextensions.sdk.cbr.v3.vault.Vault`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent vault.
        """
        return self._delete(
            _vault.Vault, vault, ignore_missing=ignore_missing,
        )

    def unbind_policy(self, vault, policy):
        """Disassociate policy from CBR vault

        :param vault: The value can be the ID of a vault
             or a :class:`~otcextensions.sdk.cbr.v3.vault.Vault`
             instance.
        :param policy: The value can be the ID of a policy
             or a :class:`~otcextensions.sdk.cbr.v3.policy.Policy`
             instance.
        """
        vault = self._get_resource(_vault.Vault, vault)
        policy = self._get_resource(_policy.Policy, policy)
        return vault.unbind_policy(
            self,
            policy_id=policy.id)

    def bind_policy(self, vault, policy):
        """Associate policy to an existing CBR vault

        :param vault: The value can be the ID of a vault
             or a :class:`~otcextensions.sdk.cbr.v3.vault.Vault`
             instance.
        :param policy: The value can be the ID of a policy
             or a :class:`~otcextensions.sdk.cbr.v3.policy.Policy`
             instance.
        """
        vault = self._get_resource(_vault.Vault, vault)
        policy = self._get_resource(_policy.Policy, policy)
        return vault.bind_policy(
            self,
            policy_id=policy.id)

    def associate_resources(self, vault, resources):
        """Associate resources to an existing CBR vault

        :param vault: The value can be the ID of a vault
             or a :class:`~otcextensions.sdk.cbr.v3.vault.Vault`
             instance.
        :param resources: array of resources in the format: id and type
        """
        vault = self._get_resource(_vault.Vault, vault)
        return vault.associate_resources(
            self,
            resources)

    def dissociate_resources(self, vault, resources):
        """Associate resources to an existing CBR vault

        :param vault: The value can be the ID of a vault
             or a :class:`~otcextensions.sdk.cbr.v3.vault.Vault`
             instance.
        :param resources: list of resource ids to be released from vault
        """
        vault = self._get_resource(_vault.Vault, vault)
        return vault.dissociate_resources(
            self,
            resources)

    # ======== Share Member ========
    def members(self, backup, **query):
        """List share members for a backup

        :param backup: The value can be the ID of a backup
            or a :class:`~otcextensions.sdk.cbr.v3.backup.Backup`
            instance.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.cbr.v3.member.Member`,
            comprised of the properties on the Member class.
        :returns: The results of config creation
        :rtype: :class:`~otcextensions.sdk.cbr.v3.member.Member`
        """
        backup = self._get_resource(_backup.Backup, backup)
        return self._list(
            _member.Member,
            backup_id=backup.id,
            **query
        )

    def get_member(self, backup, member):
        """Get one CBR share member by UUID.

        :param member: key id or an instance of
            :class:`~otcextensions.sdk.cbr.v3.member.Member`
        :param backup: The value can be the ID of a backup
            or a :class:`~otcextensions.sdk.cbr.v3.backup.Backup`
            instance.
        :returns: instance of
            :class:`~otcextensions.sdk.cbr.v3.member.Member`
        """
        backup = self._get_resource(_backup.Backup, backup)
        return self._get(
            _member.Member, member, backup_id=backup.id
        )

    def add_members(self, backup, members):
        """Add a list of share members to an existing backup

        :param backup: The value can be the ID of a backup
            or a :class:`~otcextensions.sdk.cbr.v3.backup.Backup`
            instance.
        :param list members: The list contains the project IDs of the backup
            share members to be added
        :returns: The results are the list of share member objects
        """
        backup = self._get_resource(_backup.Backup, backup)
        backup.add_members(
            self,
            members=members
        )
        members_data = []
        for member_id in members:
            member = self._get(_member.Member, member_id, backup_id=backup.id)
            members_data.append(member)
        return members_data

    def update_member(self, backup, member, status='accepted', vault=None):
        """Update CBR share members

        :param member: The id or an instance of
            :class:`~otcextensions.sdk.cbr.v3.member.Member`
        :param backup: The value can be the ID of a backup
            or a :class:`~otcextensions.sdk.cbr.v3.backup.Backup`
            instance.
        :param str status: status to be updated share member
        :param vault: The value can be the ID of a vault
             or a :class:`~otcextensions.sdk.cbr.v3.vault.Vault`
             instance.
        """
        backup = self._get_resource(_backup.Backup, backup)
        vault = self._get_resource(_vault.Vault, vault)
        return self._update(
            _member.Member, member, backup_id=backup.id,
            status=status, vault_id=vault.id)

    def delete_member(self, backup, member, ignore_missing=True):
        """Delete a single CBR share member.

        :param member: The id or an instance of
            :class:`~otcextensions.sdk.cbr.v3.member.Member`
        :param backup: The value can be the ID of a backup
            or a :class:`~otcextensions.sdk.cbr.v3.backup.Backup`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the share member does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent share member.
        """
        backup = self._get_resource(_backup.Backup, backup)
        return self._delete(
            _member.Member, member, backup_id=backup.id,
            ignore_missing=ignore_missing,
        )

    # ======== Task ========
    def tasks(self, **query):
        """List of tasks

        :param dict query: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.cbr.v3.task.Task`,
            comprised of the properties on the Task class.
        :returns: A generator of task
            :class:`~otcextensions.sdk.cbr.v3.task.Task` instances
        """
        return self._list(
            _task.Task,
            **query
        )

    def get_task(self, task):
        """Get one CBR task by id.

        :param member: key id or an instance of
            :class:`~otcextensions.sdk.cbr.v3.task.Task`
        :param task: The value can be the ID of a task
            or a :class:`~otcextensions.sdk.cbr.v3.task.Task`
            instance.
        :returns: instance of
            :class:`~otcextensions.sdk.cbr.v3.task.Task`
        """
        return self._get(_task.Task, task)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.rds.v1 import datastore


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    "id": IDENTIFIER,
    "name": "5.6.33",
    "datastore": "736270b9-27c7-4f03-823b-447d8245e1c2",
    "image": "36bdc308-0389-4830-8813-4a98d62b97de",
    "packages": "MySQL-server-5.6.33",
    "active": 1
}


class TestDatastore(base.TestCase):

    def setUp(self):
        super(TestDatastore, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sot = datastore.Datastore(**EXAMPLE)

    def test_basic(self):
        sot = datastore.Datastore()
        self.assertEqual('', sot.resource_key)
        self.assertEqual('dataStores', sot.resources_key)
        self.assertEqual('/datastores/%(datastore_name)s'
                         '/versions',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        sot = datastore.Datastore(**EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['datastore'], sot.datastore)
        self.assertEqual(EXAMPLE['image'], sot.image)
        self.assertEqual(EXAMPLE['packages'], sot.packages)
        self.assertEqual(EXAMPLE['active'], sot.active)

    def test_list(self):

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {'dataStores': [EXAMPLE]}

        self.sess.get.return_value = mock_response

        result = list(self.sot.list(
            self.sess,
            datastore_name='datastore')
        )

        self.sess.get.assert_called_once_with(
            '/datastores/%s/versions' % ('datastore'),
            params={},
        )

        self.assertEqual([datastore.Datastore(**EXAMPLE)], result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import proxy
import requests


def normalize_metric_name(name):
    name = name.replace('.', '_')
    name = name.replace(':', '_')
    return name


class Proxy(proxy.Proxy):

    def _report_stats_statsd(
            self,
            response: requests.Response | None,
            url: str | None = None,
            method: str | None = None,
            exc: BaseException | None = None,
    ) -> None:
        if not self._statsd_prefix:
            return None

        if not self._statsd_client:
            return None

        try:
            if response is not None and not url:
                url = response.request.url
            if response is not None and not method:
                method = response.request.method

            # narrow types
            assert url is not None
            assert method is not None
            assert self.service_type is not None

            name_parts = [
                normalize_metric_name(f)
                for f in self._extract_name(
                    url, self.service_type, self.session.get_project_id()
                )
            ]
            key = '.'.join(
                [
                    self._statsd_prefix,
                    normalize_metric_name(self.service_type),
                    method,
                    '_'.join(name_parts),
                ]
            )
            with self._statsd_client.pipeline() as pipe:
                if response is not None:
                    duration = int(response.elapsed.total_seconds() * 1000)
                    metric_name = f'{key}.{str(response.status_code)}'
                    pipe.timing(metric_name, duration)
                    pipe.incr(metric_name)
                    if duration > 1000:
                        pipe.incr(f'{key}.over_1000')
                elif exc is not None:
                    pipe.incr(f'{key}.failed')
                pipe.incr(f'{key}.attempted')
        except Exception:
            # We do not want errors in metric reporting ever break client
            self.log.exception("Exception reporting metrics")

    def _report_stats_influxdb(
            self,
            response: requests.Response | None,
            url: str | None = None,
            method: str | None = None,
            exc: BaseException | None = None,
    ) -> None:
        if not self._influxdb_client:
            return None

        if not self._influxdb_config:
            return None

        # NOTE(gtema): status_code is saved both as tag and field to give
        # ability showing it as a value and not only as a legend.
        # However Influx is not ok with having same name in tags and fields,
        # therefore use different names.
        if response is not None and not url:
            url = response.request.url
        if response is not None and not method:
            method = response.request.method
        tags = {
            'method': method,
            'name': '_'.join(
                [
                    normalize_metric_name(f)
                    for f in self._extract_name(
                        url, self.service_type,
                        self.session.get_project_id()
                    )
                ]
            ),
        }
        fields = {'attempted': 1}
        if response is not None:
            fields['duration'] = int(response.elapsed.total_seconds() * 1000)
            tags['status_code'] = str(response.status_code)
            # Note(gtema): emit also status_code as a value (counter)
            fields[str(response.status_code)] = 1
            fields[f'{method}.{response.status_code}'] = 1
            # Note(gtema): status_code field itself is also very helpful on the
            # graphs to show what was the code, instead of counting its
            # occurences
            fields['status_code_val'] = response.status_code
        elif exc:
            fields['failed'] = 1
        if 'additional_metric_tags' in self._influxdb_config:
            tags.update(self._influxdb_config['additional_metric_tags'])
        measurement = (
            self._influxdb_config.get('measurement', 'openstack_api')
            if self._influxdb_config
            else 'openstack_api'
        )
        # Note(gtema) append service name into the measurement name
        measurement = f'{measurement}.{self.service_type}'
        data = [{'measurement': measurement, 'tags': tags, 'fields': fields}]
        try:
            self._influxdb_client.write_points(data)
        except Exception:
            self.log.exception('Error writing statistics to InfluxDB')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestEnvironmentVar(TestApiG):
    environment = None

    def setUp(self):
        super(TestEnvironmentVar, self).setUp()
        self.create_gateway()
        self.gateway_id = TestEnvironmentVar.gateway.id
        # self.gateway_id = "be76ca6de5fe4aa7af503c03b3b44dea"
        env_attrs = {
            "name": "DEV",
            "remark": "Development environment"
        }
        environment = self.client.create_environment(
            gateway=self.gateway_id,
            **env_attrs)
        self.assertIsNotNone(environment.id)

        group_attrs = {
            "name": "api_group_001",
            "remark": "API group 1"
        }
        self.group = self.client.create_api_group(
            gateway=self.gateway_id,
            **group_attrs
        )
        self.assertIsNotNone(self.group.id)

        self.attrs = {
            "variable_name": "address",
            "variable_value": "192.168.1.5",
            "env_id": environment.id,
            "group_id": self.group.id
        }
        self.variable = self.client.create_environment_variable(
            gateway=self.gateway_id,
            **self.attrs
        )

        self.addCleanup(
            self.client.delete_environment_variable,
            gateway=self.gateway_id,
            var=self.variable
        )
        self.addCleanup(
            self.client.delete_environment,
            gateway=self.gateway_id,
            environment=environment.id,
        )
        self.addCleanup(
            self.client.delete_api_group,
            gateway=self.gateway_id,
            api_group=self.group.id,
        )
        self.addCleanup(self.delete_gateway())

    def test_list_environment_variables(self):
        vars = list(self.client.environment_variables(
            gateway=self.gateway_id,
            group_id=self.group.id))
        self.assertEqual(len(vars), 1)

    def test_get_environment_variable(self):
        var = self.client.get_environment_variable(
            gateway=self.gateway_id,
            var=self.variable.id)
        self.assertEqual(var.id, self.variable.id)

    def test_update_environment_variable(self):
        attrs = {
            "variable_value": "192.168.1.6",
        }
        updated = self.client.update_environment_variable(
            gateway=self.gateway_id,
            var=self.variable.id,
            **attrs
        )
        self.assertEqual(updated.variable_value, attrs["variable_value"])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import time

from openstack import exceptions


class RdsMixin:
    def create_rds_instance(self, name,
                            wait=True, wait_timeout=600, wait_interval=5,
                            **kwargs):
        """Create RDS instance with all the checks

        :param str availability_zone:
        :param str backup: Name or ID of the backup to create
            instance from (when from_instance is passed).
        :param int backup_keepdays:
        :param str backup_timeframe:
        :param str charge_mode: Charge mode.
        :param str configuration: dict(type=str),
        :param str datastore_type: dict(type=str, default='postgresql'),
        :param str datastore_version: dict(type=str),
        :param str disk_encryption_id: dict(type=str),
        :param str flavor: dict(required=True, type=str),
        :param str from_instance: Name or ID of the instance to create
            instance from (requires from_backup of restore_time).
        :param str ha_mode: HA mode. choices=['async', 'semisync', 'sync']
        :param str name: dict(required=True, type=str),
        :param str network: dict(type=str),
        :param str password: dict(type=str, no_log=True),
        :param int port: dict(type=int),
        :param str region: dict(type=str, choices=['eu-de'], default='eu -de'),
        :param str replica_of: dict(type=str),
        :param str restore_time: Restoration time.
        :param str router: dict(type=str),
        :param str security_group: dict(type=str),
        :param str volume_type: dict(required=True, type=str),
        :param int volume_size: dict(required=True, type=int),
        :param bool wait: dict(type=bool, default=True),
        :param int wait_timeout: dict(type=int, default=180)
        :param int wait_interval: Check interval.

        :returns: The results of server creation
        :rtype: :class:`~otcextensions.sdk.rds.v3.instance.Instance`
        """

        availability_zone = kwargs.get('availability_zone')
        backup = kwargs.get('backup')
        backup_keepdays = kwargs.get('backup_keepdays')
        backup_timeframe = kwargs.get('backup_timeframe')
        charge_mode = kwargs.get('charge_mode')
        configuration = kwargs.get('configuration')
        datastore_type = kwargs.get('datastore_type')
        datastore_version = kwargs.get('datastore_version')
        disk_encryption_id = kwargs.get('disk_encryption_id')
        flavor = kwargs.get('flavor')
        from_instance = kwargs.get('from_instance')
        ha_mode = kwargs.get('ha_mode')
        network = kwargs.get('network')
        password = kwargs.get('password')
        port = kwargs.get('port')
        region = kwargs.get('region')
        replica_of = kwargs.get('replica_of')
        restore_time = kwargs.get('restore_time')
        router = kwargs.get('router')
        security_group = kwargs.get('security_group')
        volume_type = kwargs.get('volume_type')
        volume_size = kwargs.get('volume_size')

        attrs = {}

        attrs['name'] = name

        if availability_zone:
            attrs['availability_zone'] = availability_zone
        if backup_keepdays and backup_timeframe:
            backup_attrs = {}
            backup_attrs['keep_days'] = backup_keepdays
            backup_attrs['start_time'] = backup_timeframe
            attrs['backup_strategy'] = backup_attrs
        elif backup_keepdays or backup_timeframe:
            raise exceptions.SDKException(
                '`backup_keepdays` and `backup_timeframe` must be passed'
                'together'
            )
        if charge_mode:
            attrs['charge_info'] = {'charge_mode': charge_mode}
        if configuration:
            # TODO(not_gtema): find configuration
            attrs['configuration_id'] = configuration
        if datastore_type:
            datastore = {
                'type': datastore_type,
                'version': datastore_version
            }
            attrs['datastore'] = datastore
        if disk_encryption_id:
            attrs['disk_encryption_id'] = disk_encryption_id
        if flavor:
            attrs['flavor_ref'] = flavor
        if ha_mode:
            ha = {'mode': 'ha', 'replication_mode': ha_mode}
            attrs['ha'] = ha
        if port:
            attrs['port'] = port
        if password:
            attrs['password'] = password
        if region:
            attrs['region'] = region

        volume = {}
        if volume_size:
            volume = {"size": volume_size}
            if volume_type:
                volume['type'] = volume_type
            attrs['volume'] = volume

        new_instance_required = [
            router,
            network,
            security_group,
            password
        ]

        if (not replica_of
                and not (datastore_type and datastore_version)):
            raise exceptions.SDKException(
                '`--datastore-type` and `--datastore-version` are '
                'required'
            )

        if replica_of:
            # Create replica
            if (
                password or port
                or router or security_group
                or network
            ):
                raise exceptions.SDKException(
                    'Setting password/port/router/network/sg is not '
                    'supported when creating replica'
                )
            src = self.rds.find_instance(replica_of, ignore_missing=False)
            datastore_type = src['datastore']['type']
            datastore_version = src['datastore']['version']
            attrs['replica_of_id'] = src.id
            attrs.pop('datastore', None)
        elif from_instance:
            source = self.rds.find_instance(
                from_instance, ignore_missing=False)
            if backup:
                # Create from backup
                backup_obj = self.rds.find_backup(
                    name_or_id=backup,
                    instance=source,
                    ignore_missing=False)
                attrs['restore_point'] = {
                    'type': 'backup',
                    'backup_id': backup_obj.id,
                    'instance_id': backup_obj.instance_id
                }
            elif restore_time:
                attrs['restore_point'] = {
                    'type': 'timestamp',
                    'restore_time': restore_time,
                    'instance_id': source.id
                }
        elif backup or restore_time:
            raise exceptions.SDKException(
                '`from-instance` is required when restoring from '
                'backup or using PITR.'
            )
        elif not all(new_instance_required):
            raise exceptions.SDKException(
                '`router`, `subnet`, `security-group`, '
                '`password` parameters are required when creating '
                'new primary instance.'
            )

        flavors = list(self.rds.flavors(
            datastore_name=datastore_type,
            version_name=datastore_version)
        )
        flavor_obj = None
        for f in flavors:
            if f.name == flavor:
                flavor_obj = f
        if not flavor_obj:
            raise exceptions.SDKException(
                'Flavor {flavor} can not be found'.format(
                    flavor=flavor)
            )
        if flavor_obj.instance_mode == 'ha' and not ha_mode:
            raise exceptions.SDKException(
                '`ha_mode` is required when using HA enabled flavor'
            )
        if flavor_obj.instance_mode != 'ha' and ha_mode:
            raise exceptions.SDKException(
                '`ha` enabled flavor must be '
                'chosen when setting ha_mode'
            )
        if flavor_obj.instance_mode != 'replica' and replica_of:
            raise exceptions.SDKException(
                '`replica` enabled flavor must be '
                'chosen when creating replica'
            )
        if ha_mode:
            if ',' not in availability_zone:
                raise exceptions.SDKException(
                    'List of availability zones must be used when '
                    'creating ha instance'
                )
        if ha_mode:
            mode = ha_mode
            if (datastore_type.lower() == 'postgresql'
                    and mode not in ['async', 'sync']):
                raise exceptions.SDKException(
                    '`async` or `sync` ha_mode can be used for '
                    'PostgreSQL isntance'
                )
            elif (datastore_type.lower() == 'mysql'
                    and mode not in ['async', 'semisync']):
                raise exceptions.SDKException(
                    '`async` or `semisync` ha_mode can be used for '
                    'MySQL isntance'
                )
            elif (datastore_type.lower() == 'sqlserver'
                    and mode not in ['sync']):
                raise exceptions.SDKException(
                    'Only `sync` ha_mode can be used for '
                    'SQLServer isntance'
                )
        if wait_interval and not wait:
            raise exceptions.SDKException(
                '`wait-interval` is only valid with `wait`'
            )
        if network:
            network_obj = self.network.find_network(
                network, ignore_missing=False)
            attrs['network_id'] = network_obj.id
        if security_group:
            security_group_obj = self.network.find_security_group(
                security_group, ignore_missing=False)
            attrs['security_group_id'] = security_group_obj.id
        if router:
            router_obj = self.network.find_router(router, ignore_missing=False)
            attrs['router_id'] = router_obj.id

        obj = self.rds.create_instance(**attrs)

        if obj.job_id and wait:
            wait_args = {}
            if wait_interval:
                wait_args['interval'] = wait_interval
            if wait_timeout:
                wait_args['wait'] = wait_timeout

            # RDS is so bad, that job_id appears only some time after it is
            # returned
            time.sleep(10)

            self.rds.wait_for_job(obj.job_id, **wait_args)
            obj = self.rds.get_instance(obj.id)

        return obj

    def delete_rds_instance(
        self, instance, wait=True, wait_timeout=180, wait_interval=5
    ):
        """Delete RDS instance

        :param str instance: Name, ID or instance
        :param bool wait: dict(type=bool, default=True),
        :param int wait_timeout: dict(type=int, default=180)
        :param int wait_interval: Check interval.
        """

        inst = self.rds.find_instance(instance, ignore_missing=False)

        obj = self.rds.delete_instance(inst.id)

        if obj.job_id and wait:
            wait_args = {}
            if wait_interval:
                wait_args['interval'] = wait_interval
            if wait_timeout:
                wait_args['wait'] = wait_timeout

            # RDS is so bad, that job_id appears only some time after it is
            # returned
            time.sleep(10)

            self.rds.wait_for_job(obj.job_id, **wait_args)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.swr.v2 import organization

EXAMPLE = {
    "namespace": "test_create_org_v2",
    "id": 21,
    "creator_name": "anton",
    "user_auth": 7,
}

EXAMPLE_PERMISSION = {
    "permissions": [
        {
            "user_id": "5a23ecb3999b458d92d51d524bb7fb4b",
            "user_name": "test",
            "user_auth": 1
        }
    ],
    "namespace": "test_create_org_v2"
}


class TestOrganization(base.TestCase):

    def test_basic(self):
        sot = organization.Organization()
        self.assertEqual('namespaces', sot.resources_key)
        path = '/manage/namespaces'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = organization.Organization(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['namespace'], sot.namespace)
        self.assertEqual(EXAMPLE['creator_name'], sot.creator_name)
        self.assertEqual(EXAMPLE['user_auth'], sot.user_auth)


class TestOrganizationPermissions(base.TestCase):

    def test_basic(self):
        sot = organization.Permission()
        path = '/manage/namespaces/%(namespace)s/access'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = organization.Permission(**EXAMPLE_PERMISSION)
        self.assertEqual(EXAMPLE_PERMISSION['namespace'], sot.namespace)
        self.assertEqual(
            EXAMPLE_PERMISSION['permissions'][0]["user_id"],
            sot.permissions[0].user_id
        )
        self.assertEqual(
            EXAMPLE_PERMISSION['permissions'][0]["user_auth"],
            sot.permissions[0].user_auth
        )
        self.assertEqual(
            EXAMPLE_PERMISSION['permissions'][0]["user_name"],
            sot.permissions[0].user_name
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.smn.v2 import topic


EXAMPLE = {
    "update_time": "2016-08-01T02:16:38Z",
    "push_policy": 0,
    "create_time": "2016-08-01T02:16:38Z",
    "name": "test_create_topic_v2",
    "topic_urn": "urn:smn:regionId:8bad8a40e0f7462f8c1676e3f",
    "display_name": "test create topic v2",
    "request_id": "6837531fd3f54550927b930180a706bf"
}


class TestTopic(base.TestCase):

    def test_basic(self):
        sot = topic.Topic()
        self.assertEqual('topics', sot.resources_key)
        path = '/notifications/topics'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = topic.Topic(**EXAMPLE)
        self.assertEqual(EXAMPLE['topic_urn'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['create_time'], sot.create_time)
        self.assertEqual(EXAMPLE['update_time'], sot.update_time)
        self.assertEqual(EXAMPLE['push_policy'], sot.push_policy)
        self.assertEqual(EXAMPLE['request_id'], sot.request_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Connection(resource.Resource):

    base_path = '/dcaas/direct-connects'
    resource_key = 'direct_connect'
    resources_key = 'direct_connects'

    # capabilities
    allow_list = True
    allow_fetch = True
    allow_create = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'id', 'project_id', 'name', 'description', 'port_type', 'bandwidth',
        'location', 'peer_location', 'device_id', 'interface_name',
        'redundant_id', 'provider', 'provider_status', 'type', 'hosting_id',
        'vlan', 'charge_mode', 'apply_time', 'create_time', 'delete_time',
        'order_id', 'product_id', 'status',
        'admin_state_up', project_id='tenant_id')

    # Properties
    #: ID of the direct connection.
    id = resource.Body('id')
    #: Indicates the connection name.
    name = resource.Body('name')
    #: Indicates the description of the connection.
    description = resource.Body('description')
    #: Specifies the type of the port used by the connection.
    # The value can be 1G or 10G.
    port_type = resource.Body('port_type')
    #: Specifies the connection bandwidth in Mbit/s.
    bandwidth = resource.Body('bandwidth', type=int)
    #: Specifies the connection access location.
    location = resource.Body('location')
    #: Specifies the physical location of the peer device accessed
    # by the connection, specific to the street or data center name.
    peer_location = resource.Body('peer_location')
    #: Specifies the gateway device ID of the connection.
    device_id = resource.Body('device_id')
    #: Specifies the name of the interface accessed by the connection.
    interface_name = resource.Body('interface_name')
    #: Specifies the ID of the redundant connection using the same gateway.
    redundant_id = resource.Body('redundant_id')
    #: Specifies the connection bandwidth in Mbit/s.
    provider = resource.Body('provider')
    #: Specifies the status of the carrier's leased line.
    # The value can be ACTIVE or DOWN.
    provider_status = resource.Body('provider_status')
    #: Specifies the connection type. The value can be hosted.
    type = resource.Body('type')
    #: Specifies the ID of the operations connection on which the hosted
    # connection is created.
    hosting_id = resource.Body('hosting_id')
    #: Specifies the VLAN pre-allocated to the hosted connection.
    vlan = resource.Body('vlan', type=int)
    #: Specifies the billing mode. The value can be prepayment,
    # bandwidth, or traffic.
    charge_mode = resource.Body('charge_mode')
    #: Specifies the time when the connection is requested.
    apply_time = resource.Body('apply_time')
    #: Specifies the time when the connection is created.
    create_time = resource.Body('create_time')
    #: Specifies the time when the connection is deleted.
    delete_time = resource.Body('delete_time')
    #: Specifies the order number of the connection.
    order_id = resource.Body('order_id')
    #: Specifies the product ID corresponding to the connection's order.
    product_id = resource.Body('product_id')
    #: Specifies the connection status.\
    # The value can be ACTIVE, DOWN, BUILD, ERROR, PENDING_DELETE, DELETED,
    # APPLY, DENY, PENDING_PAY, PAID, ORDERING, ACCEPT, or REJECTED.
    status = resource.Body('status')
    #: Specifies the administrative status of the connection. The value can be
    # true or false.
    admin_state_up = resource.Body('admin_state_up', type=bool)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource


class DatasetStatistics(resource.Resource):
    base_path = "/datasets/%(dataset_id)s/data-annotations/stats"

    # capabilities
    allow_fetch = True

    _query_mapping = resource.QueryParameters(
        "email",
        "locale",
        "sample_state",
    )

    # Properties
    #: Path for storing data of a dataset.
    data_path = resource.Body("data_path")
    #: Dataset ID.
    dataset_id = resource.URI("dataset_id")
    #: Deletion Stats.
    deletion_stats = resource.Body("deletion_stats", type=dict)
    #: Label statistics grouped by labeling type.
    grouped_label_stats = resource.Body("grouped_label_stats", type=dict)
    #: Statistics on hard example reasons.
    hard_detail_stats = resource.Body("hard_detail_stats", type=dict)
    #: Whether the dataset can be split into training set and validation
    #:  set based on the sample labeling statistics.
    is_data_spliting_enabled = resource.Body("data_spliting_enable", type=bool)
    #: Statistics on hard examples.
    key_sample_stats = resource.Body("key_sample_stats", type=dict)
    #: List of label statistics.
    label_stats = resource.Body("label_stats", type=list, list_type=dict)
    #: Statistics on sample metadata, in JSON format.
    metadata_stats = resource.Body("metadata_stats", type=dict)
    #: Statistics on sample status.
    sample_stats = resource.Body("sample_stats", type=dict)
